/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#include "os_interface.h"
#include "s3.h"
#include "s3g_driver.h"


/* globals constants */
const unsigned int  S3G_PAGE_SHIFT         = PAGE_SHIFT;
#ifndef __frv__
const unsigned int  S3G_PAGE_SIZE          = PAGE_SIZE;
#else
const unsigned int  S3G_PAGE_SIZE          = 0x1000;
#endif
const unsigned long S3G_PAGE_MASK          = PAGE_MASK;
const unsigned long S3G_LINUX_VERSION_CODE = LINUX_VERSION_CODE;

char *s3g_mem_track_result_path      = "/var/log/s3g-mem-track.txt";


void s3g_udelay(unsigned long long usecs_num)
{
    unsigned long long msecs = usecs_num;
    unsigned long      usecs = do_div(msecs, 1000);

    if(msecs != 0)mdelay(msecs);
    if(usecs != 0)udelay(usecs);
}

#if defined(__i386__) || defined(__x86_64__)
unsigned long long s3g_begin_timer(void)
{
    unsigned long long tsc;

    rdtscll(tsc);

    return tsc;
}

unsigned long long s3g_end_timer(unsigned long long tsc)
{
    unsigned long long curr;

    rdtscll(curr);

    return curr - tsc;
}
#endif

unsigned long long s3g_do_div(unsigned long long x, unsigned long long y)
{
    do_div(x, y);
    return x;
}

void s3g_sleep(int num)
{
    msleep(num);
}
void s3g_assert(int match)
{
   if(!match)
   {
       BUG();
   }
}

int S3G_API_CALL s3g_copy_from_user(void* to, const void* from, unsigned long size)
{
    return copy_from_user(to, from, size);
}

int S3G_API_CALL s3g_copy_to_user(void* to, const void* from, unsigned long size)
{
    return copy_to_user(to, from, size);
}


#ifndef __alpha__
static void s3g_slowbcopy(unsigned char *src, unsigned char *dst, int len)
{
#if defined(__i386__) || defined(__x86_64__)
    while(len--)
    {
        *dst++ = *src++;
        outb(0x80, 0x00);
    }
#else
    while(len--)
    {
        *dst++ = *src++;
    }
#endif
}

void s3g_slowbcopy_tobus(unsigned char *src, unsigned char *dst, int len)
{
    s3g_slowbcopy(src, dst, len);
}

void s3g_slowbcopy_frombus(unsigned char *src, unsigned char *dst, int len)
{
    s3g_slowbcopy(src, dst, len);
}
#endif

void* S3G_API_CALL s3g_memset(void* s, int c, unsigned long count)
{
    return memset(s, c, count);
}

void* S3G_API_CALL s3g_memcpy(void* d, const void* s, unsigned long count)
{
    return memcpy(d, s, count);
}

int   S3G_API_CALL s3g_memcmp(const void *s1, const void *s2, unsigned long count)
{
    return memcmp(s1, s2, count);
}

void S3G_API_CALL s3g_byte_copy(char* dst, char* src, int len)
{
#ifdef __BIG_ENDIAN__

    int i = 0;
    int left;

    for(i = 0; i < len/4; i++)
    {
        *(dst)   =  *(src+3);
        *(dst+1) =  *(src+2);
        *(dst+2) =  *(src+1);
        *(dst+3) =  *(src);
        dst     +=  4;
        src     +=  4;
    }

    left = len & 3;

    for(i = 0; i < left; i++)
    {
        *(dst+3-i) = *(src+i);
    }

#else
    s3g_memcpy(dst, src, len);
#endif
}

int   s3g_strcmp(const char *s1, const char *s2)
{
    return strcmp(s1, s2);
}

char* s3g_strcpy(char *d, const char *s)
{
    return strcpy(d, s);
}

int   s3g_strncmp(const char *s1, const char *s2, unsigned long count)
{
    return strncmp(s1, s2, count);
}

char* s3g_strncpy(char *d, const char *s, unsigned long count)
{
    return strncpy(d, s, count);
}

unsigned long s3g_strlen(char *s)
{
    return strlen(s);
}

/****************************** IO access functions*********************************/
#ifdef ENABLE_EARLYPRINTK
static void serial_outb(unsigned char value)
{
    unsigned tmt = 0xffff;

    /* by my investigate, this timeout threshold is enough for
     * serial running under 9600 baud.
     */
    while ((inb(EARLY_SERIAL_BASE + LSR) & XMTRDY) == 0 && --tmt)
    {
        CPU_RELAX();
    }

    outb(value, EARLY_SERIAL_BASE + TXR);
}

static void serial_putchar(unsigned char value)
{
    if (value == '\n')
        serial_outb('\r');

    serial_outb(value);
}

void S3G_API_CALL s3g_earlyprintk(const unsigned char *fmt, ...)
{
    static unsigned char buffer[512];
    unsigned char *index = &buffer[0];
    va_list ap;

    va_start(ap, fmt);
    vsprintf(buffer, fmt, ap);
    va_end(ap);

    while (*index)
        serial_putchar(*index++);
}
#endif

void S3G_API_CALL s3g_outb(unsigned short port, unsigned char value)
{
    outb(value, port);
}

void S3G_API_CALL s3g_outw(unsigned short port, unsigned short value)
{
    outw(value, port);
}

void S3G_API_CALL s3g_outl(unsigned short port, unsigned int value)
{
    outl(value, port);
}

char S3G_API_CALL s3g_inb(unsigned short port)
{
    return inb(port);
}

short S3G_API_CALL s3g_inw(unsigned short port)
{
    return inw(port);
}

int S3G_API_CALL s3g_inl(unsigned short port)
{
    return inl(port);
}

unsigned int S3G_API_CALL  s3g_read32(void* addr)
{
#ifndef __BIG_ENDIAN__
    return readl(addr);
#else
    register unsigned int val;
    val = *(volatile unsigned int*)(addr);
    return val;

#endif
}

unsigned short S3G_API_CALL s3g_read16(void* addr)
{
#ifndef __BIG_ENDIAN__
   return readw(addr);
#else
    register unsigned short val;
    unsigned long alignedoffset = ((unsigned long)addr/4)*4;
    unsigned long mask = (unsigned long)addr - alignedoffset;

    val = *(volatile unsigned short*)((alignedoffset+(2-mask)));
    return val;
#endif
}

unsigned char S3G_API_CALL s3g_read8(void* addr)
{
#ifndef __BIG_ENDIAN__
    return readb((void*)addr);
#else
    register unsigned char val;
    unsigned long alignedoffset = ((unsigned long)addr /4) *4;
    unsigned long mask = (unsigned long)addr - alignedoffset;
    val = *(volatile unsigned char*)((alignedoffset+(3-mask)));

    return val;
#endif
}

void S3G_API_CALL  s3g_write32(void* addr, unsigned int val)
{
#ifndef __BIG_ENDIAN__
    writel(val, addr);
#else
    *(volatile unsigned int*)(addr) = val;
#endif
}

void S3G_API_CALL  s3g_write16(void* addr, unsigned short val)
{
#ifndef __BIG_ENDIAN__
    writew(val, addr);
#else
    unsigned long alignedoffset = ((unsigned long)addr /4) *4;
    unsigned long mask = (unsigned long)addr - alignedoffset;
    *(volatile unsigned short *)((alignedoffset+(2-mask))) = (val);
#endif
}

void S3G_API_CALL  s3g_write8(void* addr, unsigned char val)
{
#ifndef __BIG_ENDIAN__
    writeb(val, addr);
#else
     unsigned long alignedoffset = ((unsigned long)addr /4) *4;
     unsigned long mask = (unsigned long)addr - alignedoffset;
     *(volatile unsigned char *)(alignedoffset+(3-mask)) = (val);
#endif
}

void S3G_API_CALL s3g_memsetio(void* addr, char c, unsigned int size )
{
#ifndef __BIG_ENDIAN__
    memset_io( addr, c, size );
#else
    int i = 0;
    int dwSize = size>>2;
    for(i = 0; i < dwSize;i++)
          s3g_write32( (unsigned char*)addr + i*4,0);
#endif
}

void S3G_API_CALL s3g_memcpy_fromio(unsigned int *dest, void *src, unsigned int size )
{
#ifndef __BIG_ENDIAN__
    memcpy_fromio( dest, src, size );
#else
    int i = 0;
    int  dwSize = size>>2;
    for(i = 0; i < dwSize;i++)
        *(dest + i) = s3g_read32((unsigned char*)src + i * 4);
#endif
}

void S3G_API_CALL s3g_memcpy_toio(void* dest, unsigned int *src, unsigned int size )
{
#ifndef __BIG_ENDIAN__
    memcpy_toio( dest, src, size );
#else
    int i = 0;
    int dwSize = size>>2;
    for(i = 0; i < dwSize;i++)
        s3g_write32((unsigned char*)dest + i*4, *(src + i));
#endif
}

struct os_file *s3g_file_open(char *path)
{
    struct os_file *file = s3g_calloc(sizeof(struct os_file));

    file->fs = get_fs();

    file->filp = filp_open(path, O_CREAT | O_WRONLY | O_APPEND, 0);
    if(file->filp == NULL)
    {
        s3g_error("open file %s failed.\n", path);
        s3g_free(file);
        file = NULL;
    }
    set_fs(KERNEL_DS);
    return file;
}

void s3g_file_close(struct os_file *file)
{
    if(file)
    {
        set_fs(file->fs);
        filp_close(file->filp, current->files);
        s3g_free(file);
    }
}

void s3g_file_read(struct os_file *file, void *buf, unsigned long size)
{
    if(file)
    {
        struct file *filp = file->filp;
        filp->f_op->read(filp, buf, size, &filp->f_pos);
    }
}

void s3g_file_write(struct os_file *file, void *buf, unsigned long size)
{
    if(file)
    {
        struct file *filp = file->filp;
        filp->f_op->write(filp, buf, size, &filp->f_pos);
    }
}

/*****************************************************************************/
int S3G_API_CALL s3g_vsprintf(char *buf, const char *fmt, ...)
{
    va_list args;
    int ret;

    va_start(args, fmt);

    ret = vsprintf(buf, fmt, args);

    va_end(args);

    return ret;
}

int S3G_API_CALL s3g_vsnprintf(char *buf, unsigned long size, const char *fmt, ...)
{
    va_list args;
    int ret;

    va_start(args, fmt);

    ret = vsnprintf(buf, size, fmt, args);

    va_end(args);

    return ret;
}

int S3G_API_CALL s3g_sscanf(char *buf, char *fmt, ...)
{
    va_list args;
    int ret;

    va_start(args, fmt);

    ret = vsscanf(buf, fmt, args);

    va_end(args);

    return ret;
}

static unsigned char* s3g_msg_prefix_string[] =
{
    "s3g debug",
    "s3g warning",
    "s3g info",
    "s3g error",
    "s3g emerg"
};

void S3G_API_CALL s3g_printk(unsigned int msglevel, const char* fmt, ...)
{
    static char buffer[512];
    va_list marker;

    if(msglevel >= S3G_MSG_LEVEL)
    {
        va_start(marker, fmt);
        vsprintf(buffer, fmt, marker);
        va_end(marker);

        switch ( msglevel )
        {
        case S3G_DRV_DEBUG:
            printk(KERN_DEBUG"%s: %s", s3g_msg_prefix_string[msglevel], buffer);
        break;
        case S3G_DRV_WARNING:
            printk(KERN_WARNING"%s: %s", s3g_msg_prefix_string[msglevel], buffer);
        break;
        case S3G_DRV_INFO:
            printk(KERN_INFO"%s: %s", s3g_msg_prefix_string[msglevel], buffer);
        break;
        case S3G_DRV_ERROR:
            printk(KERN_ERR"%s: %s", s3g_msg_prefix_string[msglevel], buffer);
        break;
        case S3G_DRV_EMERG:
            printk(KERN_EMERG"%s: %s", s3g_msg_prefix_string[msglevel], buffer);
        break;
        default:
            /* invalidate message level */
            s3g_assert(0);
        break;
        }
    }
}

#define S3G_MAX_KMALLOC_SIZE 32 * 1024

void* s3g_malloc_priv(unsigned long size)
{
    void* addr = NULL;

    if(size <= S3G_MAX_KMALLOC_SIZE)
    {
        addr = kmalloc(size, GFP_KERNEL);
    }

    if(addr == NULL)
    {
        addr = vmalloc(size);
    }

    return addr;
}

void* s3g_calloc_priv(unsigned long size)
{
    void* addr = s3g_malloc_priv(size);
    if(addr != NULL)
    {
        memset(addr, 0, size);
    }
    return addr;
}

void s3g_free_priv(void* addr)
{
    unsigned long addr_l = (unsigned long)addr;
    if(addr == NULL)  return;

#ifndef __frv__
    if(addr_l >= VMALLOC_START && addr_l < VMALLOC_END)
    {
        vfree(addr);
    }
    else
    {
        kfree(addr);
    }
#else
    kfree(addr);
#endif

}

#ifndef cmpxchg
unsigned int cmpxchg(unsigned int* ptr, unsigned int old, unsigned int new)
{
    return 0;
}
#endif

/* bit ops */
int s3g_find_first_zero_bit(void *buf, unsigned int size)
{
    return find_first_zero_bit(buf, size);
}

void s3g_set_bit(unsigned int nr, void *buf)
{
   set_bit(nr, buf);
}

void s3g_clear_bit(unsigned int nr, void *buf)
{
   clear_bit(nr, buf);
}


void s3g_getsecs(long * secs, long * usecs)
{
    struct timeval tv;

    do_gettimeofday(&tv);

    if (secs)
    {
        *secs = tv.tv_sec;
    }

    if (usecs)
    {
        *usecs= tv.tv_usec;
    }

    return;
}

static void *s3g_do_mmap(struct file *filp, s3g_map_argu_t *map)
{
    s3g_file_t    *priv     = filp->private_data;
    void          *virtual  = NULL;
    unsigned long phys_addr = 0;

    if(map->flags.mem_type == S3G_SYSTEM_IO)
    {
        phys_addr = map->phys_addr;
    }

#ifdef HAS_VM_MMAP
    priv->map = map;

    virtual = (void *)vm_mmap(filp, 0, map->size, PROT_READ | PROT_WRITE, MAP_SHARED, phys_addr);

    priv->map = NULL;
#else
    down_write(&current->mm->mmap_sem);

    priv->map = map;

    virtual = (void *)do_mmap(filp, 0, map->size, PROT_READ | PROT_WRITE, MAP_SHARED, phys_addr);

    priv->map = NULL;

    up_write(&current->mm->mmap_sem);
#endif

    return virtual;
}

static int s3g_do_munmap(s3g_vm_area_t *vma)
{
    int ret = 0;

    if(current->mm)
    {
#ifdef HAS_VM_MMAP
        ret = vm_munmap((unsigned long)vma->virt_addr, vma->size);
#else
        down_write(&current->mm->mmap_sem);

        ret = do_munmap(current->mm, (unsigned long)vma->virt_addr, vma->size);

        up_write(&current->mm->mmap_sem);
#endif
    }

    return ret;
}

void* s3g_create_thread(thread_func_t func, void *data, const char *thread_name)
{
    struct task_struct* thread;

    thread = (struct task_struct*)kthread_run(func, data, thread_name);

    return thread;
}

void s3g_destroy_thread(void* thread)
{
    if(thread)
    {
        kthread_stop(thread);
    }
}

int s3g_thread_should_stop(void)
{
    return kthread_should_stop();
}

struct os_atomic *s3g_create_atomic(int val)
{
    struct os_atomic *atomic = s3g_calloc(sizeof(struct os_atomic));

    if(atomic)
    {
        atomic_set(&atomic->counter, val);
    }

    return atomic;
}

void s3g_destroy_atomic(struct os_atomic *atomic)
{
    s3g_free(atomic);
}

int s3g_atomic_read(struct os_atomic *atomic)
{
    return atomic_read(&atomic->counter);
}

void s3g_atomic_inc(struct os_atomic *atomic)
{
    return atomic_inc(&atomic->counter);
}

void s3g_atomic_dec(struct os_atomic *atomic)
{
    return atomic_dec(&atomic->counter);
}

struct os_mutex *s3g_create_mutex(void)
{
    struct os_mutex *mutex = s3g_calloc(sizeof(struct os_mutex));

    if(mutex)
    {
        mutex_init(&mutex->lock);
    }
    return mutex;
}

void s3g_destroy_mutex(struct os_mutex *mutex)
{
    if(mutex)
    {
        s3g_free(mutex);
    }
}

int s3g_mutex_is_locked(struct os_mutex *mutex)
{
    return mutex_is_locked(&mutex->lock);
}

void s3g_mutex_show_stack(struct os_mutex *mutex)
{
#ifdef __S3G_MUTEX_DEBUG__
    kernel_show_stack(mutex->lock.owner, NULL);
#endif
}

void s3g_mutex_lock(struct os_mutex *mutex)
{
    mutex_lock(&mutex->lock);
}

int s3g_mutex_lock_killable(struct os_mutex *mutex)
{
   return mutex_lock_killable(&mutex->lock);
}

int s3g_mutex_trylock(struct os_mutex *mutex)
{
   int status = S3G_LOCKED;

   if(!mutex_trylock(&mutex->lock))
   {
       status = S3G_LOCK_FAILED;
   }

   return status;
}

void s3g_mutex_unlock(struct os_mutex *mutex)
{
    mutex_unlock(&mutex->lock);
}

struct os_sema *s3g_create_sema(int val)
{
    struct os_sema *sem = s3g_calloc(sizeof(struct os_sema));

    if(sem != NULL)
    {
        sema_init(&sem->lock, val);
    }

    return sem;
}

void s3g_destroy_sema(struct os_sema *sem)
{
    s3g_free(sem);
}

void s3g_down(struct os_sema *sem)
{
    down(&sem->lock);
}


int s3g_down_trylock(struct os_sema *sem)
{
    int status = S3G_LOCKED;

    if(down_trylock(&sem->lock))
    {
        status = S3G_LOCK_FAILED;
    }

    return status;
}

void s3g_up(struct os_sema *sem)
{
    up(&sem->lock);
}

struct os_spinlock *s3g_create_spinlock(void)
{
    struct os_spinlock *spin = s3g_calloc(sizeof(struct os_spinlock));

    if(spin != NULL)
    {
        spin_lock_init(&spin->lock);
    }

    return spin;
}

void s3g_destroy_spinlock(struct os_spinlock *spin)
{
    if(spin_is_locked(&spin->lock))
    {
        spin_unlock(&spin->lock);
    }

    s3g_free(spin);
}

void s3g_spin_lock(struct os_spinlock *spin)
{
    spin_lock(&spin->lock);
}

void s3g_spin_unlock(struct os_spinlock *spin)
{
    spin_unlock(&spin->lock);
}

unsigned long s3g_spin_lock_irqsave(struct os_spinlock *spin)
{
    unsigned long flags;

    spin_lock_irqsave(&spin->lock, flags);

    return flags;
}

void s3g_spin_unlock_irqrestore(struct os_spinlock *spin, unsigned long flags)
{
    spin_unlock_irqrestore(&spin->lock, flags);
}

struct os_wait_event* s3g_create_event()
{
    struct os_wait_event *event = s3g_calloc(sizeof(struct os_wait_event));

    if(event)
    {
        event->state = 0;
        init_waitqueue_head(&event->wait_queue);
    }

    return event;
}

void s3g_destroy_event(struct os_wait_event *event)
{
    if(event)
    {
        s3g_free(event);
    }
}

s3g_event_status_t s3g_wait_event_thread_safe(struct os_wait_event *event, condition_func_t condition, void *argu, int msec)
{
    s3g_event_status_t status = S3G_EVENT_UNKNOWN;

    if(msec)
    {
        long timeout = msecs_to_jiffies(msec);

        timeout = wait_event_timeout(event->wait_queue, (*condition)(argu), timeout);

        if(timeout > 0)
        {
            status = S3G_EVENT_BACK;
        }
        else if(timeout == 0)
        {
            status = S3G_EVENT_TIMEOUT;
        }
        else if(timeout == -ERESTARTSYS)
        {
            status = S3G_EVENT_SIGNAL;
        }
    }
    else
    {
        wait_event(event->wait_queue, (*condition)(argu));
        status = S3G_EVENT_BACK;
    }

    return status;
}

static int s3g_generic_condition_func(void *argu)
{
    struct os_wait_event *event = argu;
    int    wake = event->state;

    event->state = 0;

    return wake;
}

s3g_event_status_t s3g_wait_event(struct os_wait_event *event, int msec)
{
    return s3g_wait_event_thread_safe(event, &s3g_generic_condition_func, event, msec);
}

void s3g_wake_up_event(struct os_wait_event *event)
{
    event->state = 1;
    wake_up(&event->wait_queue);
}

s3g_event_status_t s3g_thread_wait(struct os_wait_event *event, int msec)
{
    s3g_event_status_t status = S3G_EVENT_UNKNOWN;

    if(msec)
    {
        long timeout = msecs_to_jiffies(msec);

        timeout = wait_event_interruptible_timeout(event->wait_queue,
                      s3g_generic_condition_func(event) || freezing(current) || kthread_should_stop(),
                      timeout);

        if(timeout > 0)
        {
            status = S3G_EVENT_BACK;
        }
        else if(timeout == 0)
        {
            status = S3G_EVENT_TIMEOUT;
        }
        else if(timeout == -ERESTARTSYS)
        {
            status = S3G_EVENT_SIGNAL;
        }
    }
    else
    {
        int ret = wait_event_interruptible(event->wait_queue,
                      s3g_generic_condition_func(event) || freezing(current) || kthread_should_stop());

        if(ret == 0)
        {
            status = S3G_EVENT_BACK;
        }
        else if(ret == -ERESTARTSYS)
        {
            status = S3G_EVENT_SIGNAL;
        }
    }

    return status;
}


void s3g_thread_wake_up(struct os_wait_event *event)
{
    event->state = 1;
    wake_up_interruptible(&event->wait_queue);
}

void s3g_dump_stack(void)
{
    dump_stack();
}


int  s3g_try_to_freeze(void)
{
#ifdef CONFIG_PM_SLEEP
    return try_to_freeze();
#else
    return 0;
#endif
}

int s3g_freezable(void)
{
    return !(current->flags & PF_NOFREEZE);
}

void s3g_clear_freezable(void)
{
    current->flags |= PF_NOFREEZE;
}

void s3g_old_set_freezable(void)
{
    current->flags &= ~PF_NOFREEZE;
}

void s3g_set_freezable(void)
{
#ifdef CONFIG_PM_SLEEP
    set_freezable();
#else
    current->flags &= ~PF_NOFREEZE;
#endif
}

int s3g_freezing(void)
{
    return freezing(current);
}

unsigned long s3g_get_current_pid()
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,32)
    return current->pid;
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
    return (unsigned long)current->group_leader->pids[PIDTYPE_PID].pid;
#else
    return (unsigned long)current;
#endif
}

struct os_pages_memory *s3g_allocate_pages_memory_priv(int size, alloc_pages_flags_t alloc_flags)
{
    struct os_pages_memory *memory = NULL;
    unsigned int order = 0;
    unsigned int alloc_times;
    unsigned int i, j;
    gfp_t  gfp_mask = GFP_KERNEL | __GFP_NORETRY;

    if(size <= 0)
    {
        return NULL;
    }

    /* dma32 flag only need when 64bit/PAE enabled, since only this condition CPU can use more 4G memory */
    if(alloc_flags.dma32 && (sizeof(dma_addr_t) == 8))
    {
        gfp_mask |= GFP_DMA32;
    }
    else
    {
#ifdef HAS_SET_PAGES_ARRAY_WC
        gfp_mask |= __GFP_HIGHMEM;
#endif
    }

    memory = s3g_calloc(sizeof(struct os_pages_memory));

    memory->size       = PAGE_ALIGN(size);
    memory->continuous = alloc_flags.continuous;
    memory->noswap     = alloc_flags.noswap;
    memory->dma32      = alloc_flags.dma32;
    memory->page_count = memory->size >> PAGE_SHIFT;
    memory->pages      = s3g_calloc(memory->page_count * sizeof(struct page*));
#ifdef CONFIG_X86_PAT
    memory->cache_type = S3G_MEM_WRITE_BACK;
#endif

    if(alloc_flags.continuous)
    {
        order       = get_order(memory->size);
        alloc_times = 1;
    }
    else
    {
        order       = 0;
        alloc_times = memory->page_count;
    }

    if (alloc_flags.need_clear)
        gfp_mask |= __GFP_ZERO;
    for(i = 0; i < alloc_times; i++)
    {
        memory->pages[i] = alloc_pages(gfp_mask, order);

        if(memory->pages[i] == NULL)
        {
            break;
        }
    }

    if(i < alloc_times)
    {
        for(j = 0; j < i; j++)
        {
            __free_pages(memory->pages[j], order);
        }
        s3g_free(memory->pages);
        s3g_free(memory);
        return NULL;
    }

    return memory;
}

void s3g_free_pages_memory_priv(struct os_pages_memory *memory)
{
    unsigned int  i;

    if(memory == NULL)
    {
        return;
    }

    if(memory->continuous)
    {
        __free_pages(memory->pages[0], get_order(memory->size));
    }
    else
    {
        for(i = 0; i < memory->page_count; i++)
        {
#ifdef CONFIG_X86_PAT
            if(memory->cache_type != S3G_MEM_WRITE_BACK)
            {
                os_set_page_wb(memory->pages[i]);
            }
#endif
            __free_page(memory->pages[i]);
        }
    }

    s3g_free(memory->pages);
    s3g_free(memory);
}

unsigned long long s3g_get_page_phys_address(struct os_pages_memory *memory, int page_num)
{
    unsigned long long phys_addr;

    if(memory->continuous)
    {
        phys_addr = page_to_phys(memory->pages[0]) + page_num * PAGE_SIZE;
    }
    else
    {
        phys_addr = page_to_phys(memory->pages[page_num]);
    }

    return  phys_addr;
}

struct os_pages_memory *s3g_allocate_pages_memory_dynamic(unsigned long size, alloc_pages_flags_t alloc_flags)
{
    struct os_pages_memory *memory = NULL;

    if(size < 0)
    {
        return NULL;
    }

    memory = s3g_calloc(sizeof(struct os_pages_memory));

    memory->size       = PAGE_ALIGN(size);
    memory->page_count = memory->size >> PAGE_SHIFT;
    memory->continuous = alloc_flags.continuous;
    memory->pages      = s3g_calloc(memory->page_count* sizeof(struct page*));

#ifdef DYNAMIC_MEMORY_DEBUG
    s3g_info("allocate page dynamic: size=%d, page_count=%d, origin_size=%d \n",memory->size, memory->page_count, size);
#endif
    return memory;
}

void s3g_fill_pages_memory_dynamic(struct os_pages_memory *memory_dynamic, unsigned int *index, unsigned int start, unsigned int end, struct os_pages_memory *memory)
{
    unsigned int i;
    unsigned long pfn;

#ifdef DYNAMIC_MEMORY_DEBUG
    s3g_info("fill page dynamic: index=%d, start=%d, end=%d, memory_dyanmic=%X,memory=%X \n",*index, start, end,memory_dynamic,memory);
#endif
    s3g_assert(*index+end-start<=memory_dynamic->page_count);


    pfn = page_to_pfn(memory->pages[0]);
#ifdef DYNAMIC_MEMORY_DEBUG
    s3g_info("fill page dynamic: pfn=%d,page=%x, phy_addr=%x \n", pfn, memory->pages[0], PFN_PHYS(pfn));
#endif
    for(i = 0; i <= end-start; i++)
    {
#ifdef DYNAMIC_MEMORY_DEBUG
        s3g_info("fill_page_dynamic: loop_index=%d, page=%x, phy_addr=%x \n", i, pfn_to_page(pfn+i+start), PFN_PHYS(pfn+i+start));
#endif
        memory_dynamic->pages[*index] = pfn_to_page(pfn+i+start);
        (*index)++;
    }
}

void s3g_free_pages_memory_dynamic(struct os_pages_memory *memory)
{
    if(memory == NULL)
    {
        return;
    }

    s3g_free(memory->pages);
    s3g_free(memory);
}

s3g_vm_area_t *s3g_map_pages_memory_priv(void *filp, s3g_map_argu_t *map_argu)
{
    struct os_pages_memory *memory  = map_argu->memory;
    s3g_vm_area_t          *vm_area = s3g_calloc(sizeof(s3g_vm_area_t));

#ifdef CONFIG_X86_PAT

    /* X86 use PIPT, cache based on phys pages, and when map linux will check new set cache type with old.
     * conflict will lead problem. avoid this we only set memtype when memtype not set before(treat wb as not set),
     * if memtype already set before, just use the old value. another word, we only can set memtype
     * when pages cache_type is wb if pages cache type already set like wc, uc, we just use pages value, not map's value.
     */

    if((memory->cache_type == S3G_MEM_WRITE_BACK) && (map_argu->flags.cache_type != S3G_MEM_WRITE_BACK))
    {
        unsigned int i, cache_type = map_argu->flags.cache_type;

        for(i = 0; i < memory->page_count; i++)
        {
            if(cache_type == S3G_MEM_WRITE_COMBINED)
            {
                os_set_page_wc(memory->pages[i]);
            }
            else if(cache_type == S3G_MEM_UNCACHED)
            {
                os_set_page_uc(memory->pages[i]);
            }
        }

        memory->cache_type = cache_type;
    }
    else
    {
        map_argu->flags.cache_type = memory->cache_type;
    }

#endif

    if(map_argu->flags.mem_space == S3G_MEM_KERNEL)
    {
        unsigned int cache_type = map_argu->flags.cache_type;
        pgprot_t     prot       = PAGE_KERNEL;

        prot = os_get_pgprot_val(&cache_type, prot, 0);

        vm_area->flags.value = map_argu->flags.value;
        if(map_argu->flags.mem_type == S3G_SYSTEM_RAM_DYNAMIC)
        {
            vm_area->size = map_argu->size;
        }
        else
        {
            vm_area->size        = memory->page_count * PAGE_SIZE;
        }
        vm_area->virt_addr   = vmap(memory->pages, memory->page_count, VM_MAP, prot);
    }
    else if(map_argu->flags.mem_space == S3G_MEM_USER)
    {
        vm_area->size        = map_argu->size;
        vm_area->owner       = s3g_get_current_pid();
        vm_area->virt_addr   = s3g_do_mmap(filp, map_argu);
        vm_area->flags.value = map_argu->flags.value;
    }

//    memory->ref_cnt++;

    return vm_area;
}

void s3g_unmap_pages_memory_priv(s3g_vm_area_t *vm_area)
{
    if(vm_area->flags.mem_space == S3G_MEM_KERNEL)
    {
        vunmap(vm_area->virt_addr);
    }
    else if(vm_area->flags.mem_space == S3G_MEM_USER)
    {
        s3g_do_munmap(vm_area);
    }

    s3g_free(vm_area);
}

s3g_vm_area_t *s3g_map_io_memory_priv(void *filp, s3g_map_argu_t *map_argu)
{
    s3g_vm_area_t *vm_area   = s3g_malloc(sizeof(s3g_vm_area_t));
    void          *virt_addr = NULL;

    if(map_argu->flags.mem_space == S3G_MEM_KERNEL)
    {
        if(map_argu->flags.cache_type == S3G_MEM_UNCACHED)
        {
            virt_addr = ioremap_nocache(map_argu->phys_addr, map_argu->size);
        }
        else if(map_argu->flags.cache_type == S3G_MEM_WRITE_COMBINED)
        {
#if defined(CONFIG_X86) && !defined(CONFIG_X86_PAT)
            virt_addr = ioremap_nocache(map_argu->phys_addr, map_argu->size);

            map_argu->flags.cache_type = S3G_MEM_UNCACHED;
#else
            virt_addr = ioremap_wc(map_argu->phys_addr, map_argu->size);
#endif
        }
    }
    else if(map_argu->flags.mem_space == S3G_MEM_USER)
    {
        virt_addr = s3g_do_mmap(filp, map_argu);
    }

    vm_area->flags.value = map_argu->flags.value;
    vm_area->size        = map_argu->size;
    vm_area->virt_addr   = virt_addr;
    vm_area->owner       = s3g_get_current_pid();

    return vm_area;
}

void s3g_unmap_io_memory_priv(s3g_vm_area_t *vm_area)
{
    if(vm_area->flags.mem_space == S3G_MEM_KERNEL)
    {
        iounmap(vm_area->virt_addr);
    }
    else if(vm_area->flags.mem_space == S3G_MEM_USER)
    {
        s3g_do_munmap(vm_area);
    }

    s3g_free(vm_area);
}

void s3g_flush_cache(s3g_vm_area_t *vma, void *start, unsigned int size)
{
#ifdef CONFIG_X86
    //clflush_cache_range(start, size);
    wbinvd();
#else
#if 0
    unsigned long range_start = (unsigned long)start;
    unsigned long range_end   = range_start + size;

    if(vma->flags.mem_space == S3G_MEM_USER)
    {
        struct vm_area_struct *vm_area = find_vma(current->mm, (unsigned long)vma->virt_addr);

        flush_cache_range(vm_area, range_start, range_end);
    }
    else if(vma->flags.mem_space == S3G_MEM_KERNEL)
    {
        flush_cache_vmap(range_start, range_end);
    }
#endif
#endif
}

void *s3g_ioremap(unsigned int io_base, unsigned int size)
{
    return ioremap_nocache(io_base, size);
}

void s3g_iounmap(void *map_address)
{
    iounmap(map_address);
}

int s3g_mtrr_add(unsigned long start, unsigned long size)
{
    int reg = -1;

#ifdef CONFIG_MTRR
#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 11, 0)
    reg = mtrr_add(start, size, MTRR_TYPE_WRCOMB, 1);

    if(reg < 0)
    {
         s3g_info("set mtrr range %x -> %x failed. \n", start, start + size);
    }
#else
    s3g_info("skip mtrr change. \n");
#endif
#endif

    return reg;
}

int s3g_mtrr_del(int reg, unsigned long base, unsigned long size)
{
    int err = -1;

#ifdef CONFIG_MTRR
#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 11, 0)
    err =  mtrr_del(reg, base, size);
#else
    s3g_info("skip mtrr change. \n");
#endif
#endif

    return err;
}

int s3g_get_mem_info(mem_info_t *mem)
{
    struct sysinfo si;

    si_meminfo(&si);

    /* we need set pages cache type accord usage, before set_pages_array_wc defined
     * kernel only support set cache type to normal zone pages, add check if can use hight
     */

#ifdef HAS_SET_PAGES_ARRAY_WC
    mem->totalram = si.totalram;
    mem->freeram  = si.freeram;
#else
    mem->totalram = si.totalram - si.totalhigh;
    mem->freeram  = si.freeram  - si.freehigh;
#endif

    return 0;
}

int s3g_krnl_config_file_read(const char * file_name, char *file_buffer)
{
    struct file        *filp;
    struct inode *inode;
    mm_segment_t     fs;
    off_t                fsize;

    filp = filp_open(file_name, O_RDONLY, 0);

    if(IS_ERR(filp))
    {
        s3g_error("configure file do not exist. \n");
        return 0;
    }

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,19,0)
    inode = filp->f_path.dentry->d_inode;
#else
    inode = filp->f_dentry->d_inode;
#endif
    fsize = inode->i_size;
    s3g_info("file size:%i \n", (int)fsize);

    fs = get_fs();
    set_fs(KERNEL_DS);

#if LINUX_VERSION_CODE <= KERNEL_VERSION(3,19,0)
    filp->f_op->read(filp, file_buffer, fsize, &(filp->f_pos));
#else
    vfs_read(filp, file_buffer, fsize, &(filp->f_pos));
#endif

    set_fs(fs);
    file_buffer[fsize]='\0';

    filp_close(filp, NULL);

    return 1;
}

#define CONFIG_FILE_SIZE 300
static unsigned char file_buffer[CONFIG_FILE_SIZE];
static unsigned char line_end[30] = {0};
static unsigned char opt_name[30] = {0};
int s3g_get_krnl_config_file_info(char *config_file, char *name, unsigned int *value)
{
    unsigned char i, j, k;
    unsigned int    file_size = 0;
    unsigned int    line_length;
    char               *p = NULL;

    /*read file content to file_buffer*/
    s3g_krnl_config_file_read(config_file, file_buffer);

    /*get configure file size*/
    for(i=0; i<CONFIG_FILE_SIZE; i++)
    {
        if(file_buffer[i] == '\0')
        {
            file_size = i;
            break;
        }
    }

    /*get every line end point*/
    for(i=0, j=1; i<file_size; i++)
    {
        if(file_buffer[i]=='\n')
        {
            line_end[j] = i;
            j++;
        }
    }

    for(k=1; k<j; k++)
    {
        /*get length of current line*/
        line_length = line_end[k]-line_end[k-1];

        /*point to line string start address*/
        if(k == 1)
            p = file_buffer;
        else
            p = file_buffer + line_end[k-1] + 1;

        for(i=0; i<line_length; i++)
        {
            if((*(p+i)) == '=')
            {
                /*get configure name*/
                strncpy(opt_name, p, i);

                if(!strncmp(name, opt_name, i))
                {
                    /*get configure value */
                    if(*(p+i+1) <= '9' && (*(p+i+1)) >= '0')
                    {
                        sscanf(p+i+1, "%d", value);
                        return 1;
                    }
                }
            }
        }
    }

    /*name not found in configure file*/
    return 0;
}

void s3g_debug_file_read(void)
{

    unsigned int config_value;
    unsigned int timeout = 8000; // sleep at least 8 secs
    char *name = "trap_me";
    char *config_file = "/etc/X11/via_kernel.conf";

    do {

        config_value = 0;

        s3g_get_krnl_config_file_info(config_file, name, &config_value);

        s3g_info("%s: %d\n", __func__, config_value);

        s3g_sleep(timeout);

    } while (!config_value);

    s3g_info("%s: save you another 10 secs for next breakpoints\n", __func__);

    s3g_sleep(10000);

    return;
}
