/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#ifndef __OS_INTERFACE_H__
#define __OS_INTERFACE_H__

#if !defined(S3G_API_CALL)
#define S3G_API_CALL
#endif

#ifndef TRUE
#define TRUE      1
#define FALSE     0
#endif

#ifndef NULL
#define NULL ((void*)0)
#endif

#define S3G_DRV_DEBUG     0x00
#define S3G_DRV_WARNING   0x01
#define S3G_DRV_INFO      0x02
#define S3G_DRV_ERROR     0x03
#define S3G_DRV_EMERG     0x04

/********** mem track ******/
#define S3G_MALLOC_TRACK       0
#define S3G_ALLOC_PAGE_TRACK   0
#define S3G_MAP_PAGES_TRACK    0
#define S3G_MAP_IO_TRACK       0

#define S3G_CHECK_MALLOC_OVERFLOW       0

#define S3G_MEM_TRACK_RESULT_TO_FILE    0


typedef struct 
{
    unsigned short  vendor_id; 
    unsigned short  device_id;    
    unsigned short  command;       
    unsigned short  status;
    unsigned char   revision_id; 
    unsigned char   prog_if;     
    unsigned char   sub_class;  
    unsigned char   base_class;     
    unsigned char   cache_line_size;  
    unsigned char   latency_timer;        
    unsigned char   header_type;              
    unsigned char   bist;
    unsigned short  sub_sys_vendor_id;
    unsigned short  sub_sys_id;
    unsigned short  link_status;          
    unsigned int    start_addr[7];
    unsigned int    end_addr[7];
} bus_config_t;


typedef struct
{
    unsigned long totalram; //s3g can used system pages num
    unsigned long freeram;
}mem_info_t;

/*****************************************************************************/
typedef struct
{
    void *pdev;
}os_device_t;

typedef int (*condition_func_t)(void *argu);

extern void s3g_udelay(unsigned long long usecs_num);

extern unsigned long long s3g_begin_timer(void);
extern unsigned long long s3g_end_timer(unsigned long long);

extern unsigned long long s3g_do_div(unsigned long long x, unsigned long long y);

extern int  s3g_find_first_zero_bit(void *buf, unsigned int size);
extern void s3g_set_bit(unsigned int nr, void *buf);
extern void s3g_clear_bit(unsigned int nr, void *buf);

extern struct os_atomic *s3g_create_atomic(int val);
extern void s3g_destroy_atomic(struct os_atomic *atomic);
extern int  s3g_atomic_read(struct os_atomic *atomic);
extern void s3g_atomic_inc(struct os_atomic *atomic);
extern void s3g_atomic_dec(struct os_atomic *atomic);

#define S3G_LOCKED       0
#define S3G_LOCK_FAILED  1

extern struct os_mutex *s3g_create_mutex(void);
extern void  s3g_destroy_mutex(struct os_mutex *mutex);
extern int s3g_mutex_is_locked(struct os_mutex *mutex);
extern void s3g_mutex_show_stack(struct os_mutex *mutex);
extern void  s3g_mutex_lock(struct os_mutex *mutex);
extern int   s3g_mutex_lock_killable(struct os_mutex *mutex);
extern int   s3g_mutex_trylock(struct os_mutex *mutex);
extern void  s3g_mutex_unlock(struct os_mutex *mutex);

extern struct os_sema *s3g_create_sema(int value);
extern void  s3g_destroy_sema(struct os_sema *sema);
extern void  s3g_down(struct os_sema *sema);
extern int   s3g_down_trylock(struct os_sema *sema);  
extern void  s3g_up(struct os_sema *sema);

extern struct os_spinlock *s3g_create_spinlock(void);
extern void   s3g_destroy_spinlock(struct os_spinlock *spin);
extern void   s3g_spin_lock(struct os_spinlock *spin);
extern void   s3g_spin_unlock(struct os_spinlock *spin);
extern unsigned long s3g_spin_lock_irqsave(struct os_spinlock *spin);
extern void   s3g_spin_unlock_irqrestore(struct os_spinlock *spin, unsigned long flags);

extern int S3G_API_CALL s3g_copy_from_user(void *to, const void *from, unsigned long size);
extern int S3G_API_CALL s3g_copy_to_user(void *to, const void *from, unsigned long size);

extern void* S3G_API_CALL s3g_memset(void *s, int c, unsigned long count);
extern void* S3G_API_CALL s3g_memcpy(void *d, const void *s, unsigned long count);
extern int   S3G_API_CALL s3g_memcmp(const void *, const void *, unsigned long);
extern void  S3G_API_CALL s3g_byte_copy(char* dst, char* src, int len);

extern int   s3g_strcmp(const char *, const char *);
extern char* s3g_strcpy(char *, const char *);
extern int   s3g_strncmp(const char *, const char *, unsigned long);
extern char* s3g_strncpy(char *, const char *, unsigned long);
extern unsigned long s3g_strlen(char *s);

extern void  s3g_slowbcopy_tobus(unsigned char *src, unsigned char *dst, int len);
extern void  s3g_slowbcopy_frombus(unsigned char *src, unsigned char *dst, int len);

extern void s3g_sleep(int);

extern unsigned long s3g_get_current_pid(void);
/**/
#if defined(__i386__) || defined(__x86_64__)
#define s3g_mb()       asm volatile("mfence":::"memory")
#define s3g_rmb()      asm volatile("lfence":::"memory")
#define s3g_wmb()      asm volatile("sfence":::"memory") 
#define s3g_flush_wc() s3g_wmb()
#else
#if __LINUX_ARM_ARCH__ >= 7
#define dmb() __asm__ __volatile__ ("dmb" : : : "memory")

#define s3g_mb()    dmb()
#define s3g_rmb()   dmb()
#define s3g_wmb()   dmb()
#define s3g_flush_wc() dmb()
#endif
#endif



/****************************** IO access functions*********************************/
#define ENABLE_EARLYPRINTK

#ifdef ENABLE_EARLYPRINTK
#define TTYS0_IO_BASE         0x03f8
#define TTYS1_IO_BASE         0x02f8
/* note, we use ttyS0 by default */
#define EARLY_SERIAL_BASE     TTYS0_IO_BASE
#define LSR                   5
#define TXR                   0
#define XMTRDY                0x20
#define CPU_RELAX()           asm volatile("rep;nop")

extern void S3G_API_CALL s3g_earlyprintk(const unsigned char *fm, ...);
#endif
extern void S3G_API_CALL s3g_outb(unsigned short port, unsigned char value);
extern void S3G_API_CALL s3g_outw(unsigned short port, unsigned short value);
extern void S3G_API_CALL s3g_outl(unsigned short port, unsigned int value);
extern char S3G_API_CALL s3g_inb(unsigned short port);
extern short S3G_API_CALL s3g_inw(unsigned short port);
extern int S3G_API_CALL s3g_inl(unsigned short port);
extern unsigned int S3G_API_CALL  s3g_read32(void *addr);
extern unsigned short S3G_API_CALL s3g_read16(void *addr);
extern unsigned char S3G_API_CALL s3g_read8(void *addr);
extern void S3G_API_CALL  s3g_write32(void *addr, unsigned int val);
extern void S3G_API_CALL  s3g_write16(void *addr, unsigned short val);
extern void S3G_API_CALL  s3g_write8(void *addr, unsigned char val);
extern void S3G_API_CALL s3g_memsetio(void *addr, char c, unsigned int size);
extern void S3G_API_CALL s3g_memcpy_fromio(unsigned int *dest, void *src, unsigned int size);
extern void S3G_API_CALL s3g_memcpy_toio(void *dest, unsigned int *src, unsigned int size);

extern int S3G_API_CALL s3g_vsprintf(char *buf, const char *fmt, ...);
extern int S3G_API_CALL s3g_vsnprintf(char *buf, unsigned long size, const char *fmt, ...);

extern int S3G_API_CALL s3g_sscanf(char *buf, char *fmt, ...);

extern void s3g_dump_stack(void);
extern void S3G_API_CALL s3g_assert(int a);
extern void S3G_API_CALL s3g_printk(unsigned int msglevel, const char* fmt, ...);

#define s3g_16byte_align(size) ((size + 15)&0xfffffff0)

#ifdef _DEBUG_
#define S3G_MSG_LEVEL S3G_DRV_DEBUG
#define s3g_debug(args...) s3g_printk(S3G_DRV_DEBUG, ##args)
#else
#define S3G_MSG_LEVEL S3G_DRV_INFO
#define s3g_debug
#endif

#define s3g_emerg(args...)   s3g_printk(S3G_DRV_EMERG, ##args)
#define s3g_error(args...)   s3g_printk(S3G_DRV_ERROR, ##args)
#define s3g_info(args...)    s3g_printk(S3G_DRV_INFO,  ##args)
#define s3g_warning(args...) s3g_printk(S3G_DRV_WARNING, ##args)

extern void  s3g_getsecs(long * secs, long * usecs);

typedef int (*thread_func_t)(void *data);

extern void* s3g_create_thread(thread_func_t func, void *data, const char *thread_name);
extern void s3g_destroy_thread(void *thread);
extern int  s3g_thread_should_stop(void);

typedef enum
{
    S3G_EVENT_UNKNOWN = 0,
    S3G_EVENT_BACK    = 1, /* condtion meet event back */
    S3G_EVENT_TIMEOUT = 2, /* wait timeout */
    S3G_EVENT_SIGNAL  = 3, /* wait interrupt by a signal */
}s3g_event_status_t;

extern struct os_wait_event* s3g_create_event(void);
extern void s3g_destroy_event(struct os_wait_event *event);
extern void s3g_wake_up_event(struct os_wait_event *event);
extern s3g_event_status_t s3g_wait_event(struct os_wait_event *event, int msec);
extern void s3g_thread_wake_up(struct os_wait_event *event);
extern s3g_event_status_t s3g_thread_wait(struct os_wait_event *event, int msec);
extern s3g_event_status_t s3g_wait_event_thread_safe(struct os_wait_event *event, condition_func_t condition, void *argu, int msec);

extern struct os_file *s3g_file_open(char *path);
extern void s3g_file_close(struct os_file *file);
extern void s3g_file_read(struct os_file *file, void *buf, unsigned long size);
extern void s3g_file_write(struct os_file *file, void *buf, unsigned long size);

extern int  s3g_try_to_freeze(void);
extern void s3g_old_set_freezable(void);
extern void s3g_set_freezable(void);
extern void s3g_clear_freezable(void);
extern int  s3g_freezing(void);
extern int  s3g_freezable(void);

enum s3g_mem_type
{
    S3G_SYSTEM_IO             = 0x01,
    S3G_SYSTEM_RAM            = 0x02,
    S3G_SYSTEM_RAM_DYNAMIC    = 0x04,
};

enum s3g_mem_space
{
    S3G_MEM_KERNEL = 0x01,
    S3G_MEM_USER   = 0x02,
};

enum s3g_mem_attribute
{   
    S3G_MEM_WRITE_BACK     = 0x01,
    S3G_MEM_WRITE_THROUGH  = 0x02,
    S3G_MEM_WRITE_COMBINED = 0x03,
    S3G_MEM_UNCACHED       = 0x04,
};

typedef struct 
{
    unsigned int continuous  :1;
    unsigned int noswap      :1;
    unsigned int dma32       :1;
    unsigned int need_clear  :1;
    unsigned int reserved    :28;
}alloc_pages_flags_t;

typedef struct
{
    union
    {
        struct 
        {
            unsigned char mem_type;
            unsigned char mem_space;
            unsigned char cache_type;
            unsigned char read_only  :1;
            unsigned char write_only :1;
        };
        unsigned int  value;
    };
}s3g_map_flags_t;

typedef struct
{
    s3g_map_flags_t flags;
    union
    {
        struct os_pages_memory *memory;
        unsigned long          phys_addr;
    };
    unsigned long size;
}s3g_map_argu_t;

typedef struct __s3g_vm_area
{
    s3g_map_flags_t flags;
    unsigned long   size;
    unsigned long   owner;
    unsigned int    need_flush_cache;
    void            *virt_addr;
    struct __s3g_vm_area *next;
}s3g_vm_area_t;


/* os private alloc pages func*/
extern struct os_pages_memory* s3g_allocate_pages_memory_priv(int size, alloc_pages_flags_t alloc_flags);
extern void s3g_free_pages_memory_priv(struct os_pages_memory *memory);
extern s3g_vm_area_t *s3g_map_pages_memory_priv(void *process_context, s3g_map_argu_t *map);
extern void s3g_unmap_pages_memory_priv(s3g_vm_area_t *vm_area);
extern s3g_vm_area_t *s3g_map_io_memory_priv(void *process_context, s3g_map_argu_t *map);
extern void s3g_unmap_io_memory_priv(s3g_vm_area_t *map);
extern unsigned long long s3g_get_page_phys_address(struct os_pages_memory *memory, int page_num);

extern void s3g_flush_cache(s3g_vm_area_t *vma, void *start, unsigned int size);
extern int  s3g_mtrr_add(unsigned long base, unsigned long size);
extern int  s3g_mtrr_del(int reg, unsigned long base, unsigned long size);

/* os private malloc func */
extern void* s3g_malloc_priv(unsigned long size);
extern void* s3g_calloc_priv(unsigned long size);
extern void  s3g_free_priv(void *addr);
/* os dynamic pages memory func */
extern struct os_pages_memory *s3g_allocate_pages_memory_dynamic(unsigned long size, alloc_pages_flags_t alloc_flags);
extern void s3g_fill_pages_memory_dynamic(struct os_pages_memory *memory_dynamic, unsigned int *index, unsigned int start, unsigned int end, struct os_pages_memory *memory);
extern void s3g_free_pages_memory_dynamic(struct os_pages_memory *memory);

extern int s3g_get_command_status16(void *dev, unsigned short *command);
extern int s3g_get_command_status32(void *dev, unsigned int *command);
extern int s3g_write_command_status16(void *dev, unsigned short command);
extern int s3g_write_command_status32(void *dev, unsigned int command);
extern int s3g_get_rom_save_addr(void *dev, unsigned int *romsave);
extern int s3g_write_rom_save_addr(void *dev, unsigned int romsave);
extern int s3g_get_bar1(void *dev, unsigned int *bar1);

extern void s3g_get_bus_config(void *pdev, bus_config_t *bus);

extern unsigned long s3g_get_rom_start_addr(void *dev);
extern void         *s3g_ioremap( unsigned int io_base, unsigned int size);
extern void          s3g_iounmap(void *map_address);
/* mem track func*/
extern void s3g_mem_track_init(void);
extern void s3g_mem_track_list_result(void);

/* alloc pages track func*/
extern struct os_pages_memory *s3g_allocate_pages_memory_track(int size, 
                                   alloc_pages_flags_t alloc_flags, 
                                   char *file, unsigned int line);
extern void s3g_free_pages_memory_track(struct os_pages_memory *memory, 
                                        char *file, unsigned int line);

extern s3g_vm_area_t *s3g_map_pages_memory_track(void *process_context, 
                                        s3g_map_argu_t *map, 
                                        char *file, unsigned int line);
extern void s3g_unmap_pages_memory_track(s3g_vm_area_t *vm_area, 
                                         char *file, unsigned int line);

extern s3g_vm_area_t *s3g_map_io_memory_track(void *process_context, 
                                     s3g_map_argu_t *map,
                                     char *file, unsigned int line);
extern void s3g_unmap_io_memory_track(s3g_vm_area_t *vm_area,
                                      char *file, unsigned int line);

/* malloc track func*/
extern void* s3g_malloc_track(unsigned long size, char *file, unsigned int line);
extern void* s3g_calloc_track(unsigned long size, char *file, unsigned int line);
extern void  s3g_free_track(void *addr, char *file, unsigned int line);

extern void  s3g_mem_leak_list(void);

#if S3G_MALLOC_TRACK
#define s3g_malloc(size)         s3g_malloc_track(size, __FILE__, __LINE__)
#define s3g_calloc(size)         s3g_calloc_track(size, __FILE__, __LINE__)
#define s3g_free(addr)           s3g_free_track(addr, __FILE__, __LINE__)
#else
#define s3g_malloc(size)         s3g_malloc_priv(size)
#define s3g_calloc(size)         s3g_calloc_priv(size)
#define s3g_free(addr)           s3g_free_priv(addr)
#endif

#if S3G_ALLOC_PAGE_TRACK
#define s3g_allocate_pages_memory(size, flag) \
        s3g_allocate_pages_memory_track(size, flag, __FILE__, __LINE__)
#define s3g_free_pages_memory(memory)         \
        s3g_free_pages_memory_track(memory, __FILE__, __LINE__) 
#else
#define s3g_allocate_pages_memory(size, flag) \
        s3g_allocate_pages_memory_priv(size, flag)
#define s3g_free_pages_memory(memory)         \
        s3g_free_pages_memory_priv(memory)
#endif

#if S3G_MAP_PAGES_TRACK
#define s3g_map_pages_memory(priv, argu)     \
        s3g_map_pages_memory_track(priv, argu, __FILE__, __LINE__)
#define s3g_unmap_pages_memory(map) \
        s3g_unmap_pages_memory_track(map, __FILE__, __LINE__)
#else
#define s3g_map_pages_memory(priv, argu)     \
        s3g_map_pages_memory_priv(priv, argu)
#define s3g_unmap_pages_memory(vma) \
        s3g_unmap_pages_memory_priv(vma)
#endif

#if S3G_MAP_IO_TRACK
#define s3g_map_io_memory(priv, argu) \
        s3g_map_io_memory_track(priv, argu, __FILE__, __LINE__)
#define s3g_unmap_io_memory(argu) \
        s3g_unmap_io_memory_track(argu, __FILE__, __LINE__)
#else
#define s3g_map_io_memory(priv, argu) \
        s3g_map_io_memory_priv(priv, argu)
#define s3g_unmap_io_memory(argu) \
        s3g_unmap_io_memory_priv(argu)
#endif

extern int s3g_get_mem_info(mem_info_t *mem);
extern int s3g_krnl_config_file_read(const char * file_name, char *file_buffer);
extern int s3g_get_krnl_config_file_info(char *config_file, char *name, unsigned int *value);
extern void s3g_debug_file_read(void);

/* global constants */
extern const unsigned int   S3G_PAGE_SHIFT;
extern const unsigned int   S3G_PAGE_SIZE;
extern const unsigned long  S3G_PAGE_MASK;
extern char *s3g_mem_track_result_path;

#endif

