/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#ifndef __S3G_H__
#define __S3G_H__

#ifdef __KERNEL__
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/time.h>
#include <linux/interrupt.h>
#include <linux/miscdevice.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/file.h>
#include <linux/pci.h>
#include <linux/leds.h>
#include <linux/platform_device.h>
#include <linux/console.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/kthread.h>
#include <linux/delay.h>
#include <linux/mm.h>
#include <linux/pagemap.h>
#include <linux/vmalloc.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <linux/string.h>

//#define __S3G_MUTEX_DEBUG__

#ifdef __S3G_MUTEX_DEBUG__
#include <linux/kallsyms.h>
#endif

#if defined(__alpha__) || defined(__powerpc__)
#include <asm/pgtable.h>    /* For pte_wrprotect */
#endif

#ifdef __alpha__
#include <asm/current.h>
#endif

#include <asm/io.h>
#include <asm/mman.h>
#include <asm/uaccess.h>
#include <asm/cacheflush.h>
#include <asm/pgalloc.h>
#ifdef CONFIG_MTRR
#include <asm/mtrr.h>
#endif

#if defined(CONFIG_VGA_ARB)
#include <linux/vgaarb.h>
#endif
#endif

#ifdef CONFIG_FB
#include <linux/fb.h>
#endif

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
#endif

#ifndef DRM_VERSION_CODE
#define DRM_VERSION_CODE LINUX_VERSION_CODE
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,39)
#include <linux/smp_lock.h>    /* For (un)lock_kernel */
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
#include <linux/freezer.h>
#endif

#if defined(__i386__) || defined(__x86_64__)
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,2,0)
#include <asm/i387.h>
#endif
#endif
#include <asm/div64.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,41)
#include <linux/tqueue.h>
#else
#include <linux/workqueue.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,35) 
#define HAS_SET_PAGES_ARRAY_WC
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
#define HAS_VM_MMAP
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)
#define cap_t(x) (x).cap[0]
#endif

#ifndef IRQF_SHARED
#define IRQF_SHARED SA_SHIRQ
#endif

#ifndef PM_EVENT_SLEEP
#define PM_EVENT_SLEEP PM_EVENT_SUSPEND
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,38) 
#define acquire_console_sem console_lock
#define release_console_sem console_unlock
#endif

struct os_atomic
{
    atomic_t counter; 
};

struct os_spinlock
{
    spinlock_t lock;
};

struct os_sema
{
    struct semaphore lock;
};


#ifndef mutex_init
struct os_mutex
{
    struct semaphore lock;
};
#define mutex_init(lock)		sema_init(lock, 1)
#define mutex_destroy(lock)		sema_init(lock, -99)
#define mutex_lock(lock)		down(lock)
#define mutex_trylock(lock)		(down_trylock(lock) ? 0 : 1)
#define mutex_unlock(lock)		up(lock)
#define mutex_lock_killable(lock)       down_killable(lock)
#else
struct os_mutex
{
    struct mutex lock;
};
#endif

struct os_file
{
    mm_segment_t fs;
    struct file  *filp;
};


struct os_pages_memory
{
    unsigned int size;
    unsigned int page_count;
    unsigned int cache_type:8;
    unsigned int continuous:1;
    unsigned int noswap:1;
    unsigned int dma32:1; //alloc in low 4G phys addr
    unsigned int ref_cnt;
    struct page **pages;
};

struct os_wait_event
{
    int                state;
    wait_queue_head_t  wait_queue;
};

#ifdef CONFIG_X86_PAT

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,19,0)
/*
 * X86 PAT uses page flags WC and Uncached together to keep track of
 * memory type of pages that have backing page struct. X86 PAT supports 3
 * different memory types, _PAGE_CACHE_MODE_WB, _PAGE_CACHE_MODE_WC and
 * _PAGE_CACHE_MODE_UC_MINUS and fourth state where page's memory type has not
 * been changed from its default (value of -1 used to denote this).
 * Note we do not support _PAGE_CACHE_MODE_UC here.
 */

#define _PGMT_DEFAULT		0
#define _PGMT_WC		    (1UL << PG_arch_1)
#define _PGMT_UC_MINUS		(1UL << PG_uncached)
#define _PGMT_WB		    (1UL << PG_uncached | 1UL << PG_arch_1)
#define _PGMT_MASK		    (1UL << PG_uncached | 1UL << PG_arch_1)
#define _PGMT_CLEAR_MASK	(~_PGMT_MASK)

static inline enum page_cache_mode get_page_memtype(struct page *pg)
{
	unsigned long pg_flags = pg->flags & _PGMT_MASK;

	if (pg_flags == _PGMT_DEFAULT)
		return -1;
	else if (pg_flags == _PGMT_WC)
		return _PAGE_CACHE_MODE_WC;
	else if (pg_flags == _PGMT_UC_MINUS)
		return _PAGE_CACHE_MODE_UC_MINUS;
	else
		return _PAGE_CACHE_MODE_WB;
}

static inline void set_page_memtype(struct page *pg,
				    enum page_cache_mode memtype)
{
	unsigned long memtype_flags;
	unsigned long old_flags;
	unsigned long new_flags;

	switch (memtype) {
	case _PAGE_CACHE_MODE_WC:
		memtype_flags = _PGMT_WC;
		break;
	case _PAGE_CACHE_MODE_UC_MINUS:
		memtype_flags = _PGMT_UC_MINUS;
		break;
	case _PAGE_CACHE_MODE_WB:
		memtype_flags = _PGMT_WB;
		break;
	default:
		memtype_flags = _PGMT_DEFAULT;
		break;
	}

	do {
		old_flags = pg->flags;
		new_flags = (old_flags & _PGMT_CLEAR_MASK) | memtype_flags;
	} while (cmpxchg(&pg->flags, old_flags, new_flags) != old_flags);
}
#endif

static inline int os_set_page_uc(struct page *pg)
{
    int retval;
#ifdef HAS_SET_PAGES_ARRAY_WC
    retval = set_pages_array_uc(&pg, 1);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,19,0)
        if(get_page_memtype(pg) != _PAGE_CACHE_MODE_UC)
        {
            set_page_memtype(pg, _PAGE_CACHE_MODE_UC);
        }
#else
        if(get_page_memtype(pg) != _PAGE_CACHE_UC)
        {
            set_page_memtype(pg, _PAGE_CACHE_UC);
        }
#endif

#else
    retval = set_memory_uc((unsigned long)page_address(pg), 1);
#endif

    if(retval)
    {
        printk(KERN_ERR "s3g" "set page to uc failed.\n");
    }

    return retval;
}

static inline int os_set_page_wc(struct page *pg)
{
    int retval;
#ifdef HAS_SET_PAGES_ARRAY_WC
    retval = set_pages_array_wc(&pg, 1);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,19,0)
    if(get_page_memtype(pg) != _PAGE_CACHE_MODE_WC)
    {
        set_page_memtype(pg, _PAGE_CACHE_MODE_WC);
    }
#else
    if(get_page_memtype(pg) != _PAGE_CACHE_WC)
    {
        set_page_memtype(pg, _PAGE_CACHE_WC);
    }
#endif
#else
    retval = set_memory_wc((unsigned long)page_address(pg), 1);
#endif

    if(retval)
    {
        printk(KERN_ERR "s3g" "set page to wc failed.\n");
    }

    return retval;
}

static inline int os_set_page_wb(struct page *pg)
{
    int retval;
#ifdef HAS_SET_PAGES_ARRAY_WC
    retval = set_pages_array_wb(&pg, 1);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,19,0)
    if(get_page_memtype(pg) != _PAGE_CACHE_MODE_WB)
    {
        set_page_memtype(pg, _PAGE_CACHE_MODE_WB);
    }
#else
    if(get_page_memtype(pg) != _PAGE_CACHE_WB)
    {
        set_page_memtype(pg, _PAGE_CACHE_WB);
    }
#endif
#else
    retval = set_memory_wb((unsigned long)page_address(pg), 1);
#endif

    if(retval)
    {
        printk(KERN_ERR "s3g" "set page to wb failed.\n");
    }

    return retval;
}
#endif

#ifdef __S3G_MUTEX_DEBUG__
typedef unsigned long (*kernel_show_stack_t)(struct task_struct *, unsigned long *);
extern kernel_show_stack_t kernel_show_stack;
#endif

extern pgprot_t os_get_pgprot_val(unsigned int *cache_type, pgprot_t old_prot, int io_map);

#endif

