/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#include "s3.h"
#include "s3g_driver.h"
#include "os_interface.h"
#include "s3g_version.h"

#include <drm/drmP.h>
#include <linux/console.h>
#include <linux/module.h>

#if ! defined(LINUX_VERSION_CODE)  
#include <linux/version.h>
#endif

#define DRIVER_NAME     "s3g_drm"
#define DRIVER_DESC     "S3G DRM Pro"

int num_probed_card = 0;

static struct pci_device_id pciidlist[] =
{
    {0x5333, 0x8e00, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8e01, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8e02, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8e03, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8e04, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8e05, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8e06, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8e07, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E10, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E12, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E11, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E13, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E20, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E49, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E48, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E47, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E45, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E40, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E4A, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E4B, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E4C, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x8E4F, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x900F, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x902F, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x905F, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9020, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9021, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9022, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9023, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9024, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9025, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9033, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9034, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9042, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9043, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9044, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9045, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9050, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9053, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9054, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9060, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x9070, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x1106, 0x6122, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
    {0x5333, 0x110F, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},    
    {0x1106, 0x3A01, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},    
    {0, 0, 0}
};

#ifdef __mips__
int s3g_swfb = 1;
#else
int s3g_swfb = 0;
#endif
char *s3g_fb_mode = NULL; //"800x600-32@60";
int s3g_fb   = 1;
int s3g_flip = 0;
s3g_card_t s3g_cards[S3G_MAX_CARDS];

#if (DRM_VERSION_CODE < 0x40800)
int  via_chrome9_drm_resume(struct drm_device *dev);
int  via_chrome9_drm_suspend(struct drm_device *dev, pm_message_t state);

int  via_chrome9_drm_resume(struct drm_device *dev)
{
	return 0;
}

int  via_chrome9_drm_suspend(struct drm_device *dev, pm_message_t state)
{
	return 0;
}
#endif

struct file_operations s3g_drm_fops = {
    .owner = THIS_MODULE,
    .open = drm_open,
    .release = drm_release,
    .unlocked_ioctl = drm_ioctl,
    .mmap = NULL,
    .poll = drm_poll,
#ifdef HAS_drm_fasync
    .fasync = drm_fasync,
#endif
    .read = drm_read,
    .llseek = noop_llseek,
};

static struct drm_driver driver = {
	.driver_features =
		DRIVER_HAVE_DMA | DRIVER_GEM |
#ifdef HAS_DRIVER_FB_DMA
		DRIVER_FB_DMA |
#endif
#ifdef HAS_DRIVER_USE_MTRR
		DRIVER_USE_MTRR |
#endif
		DRIVER_HAVE_IRQ | DRIVER_IRQ_SHARED,
	.load = NULL,
	.preclose = NULL,
	.lastclose = NULL,
	.unload = NULL,
#if (DRM_VERSION_CODE < 0x40800)
	.suspend = via_chrome9_drm_suspend,
	.resume = via_chrome9_drm_resume,
#endif
#if (DRM_VERSION_CODE < KERNEL_VERSION(4,14,0) && DRM_VERSION_CODE >= KERNEL_VERSION(3,18,0))
    .set_busid = drm_pci_set_busid,
#endif
	.irq_preinstall = NULL,
	.irq_postinstall = NULL,
	.irq_uninstall = NULL,
	.irq_handler = NULL,
#if (DRM_VERSION_CODE >= KERNEL_VERSION(4,10,0))
	.get_vblank_counter = NULL,
#else
	.get_vblank_counter = drm_vblank_count,
#endif
	.enable_vblank = NULL,
	.disable_vblank = NULL,
#if (DRM_VERSION_CODE <= KERNEL_VERSION(3,5,35))
	.reclaim_buffers = drm_core_reclaim_buffers,
#endif
#ifdef HAS_gem_init_object
	.gem_init_object = NULL,
#endif
	.gem_free_object = NULL,
	.ioctls = NULL,
#if (DRM_VERSION_CODE < KERNEL_VERSION(3,2,50))
	.fops = {
		 .owner = THIS_MODULE,
		 .open = drm_open,
		 .release = drm_release,
		 .unlocked_ioctl = drm_ioctl,
		 .mmap = NULL,
		 .poll = drm_poll,
		 .fasync = drm_fasync,
		 .read = drm_read,
		 .llseek = noop_llseek,
	},
#else
    .fops = &s3g_drm_fops,
#endif
#if !(DRM_VERSION_CODE >= 0x030000)
	.pci_driver = {
		 .name = DRIVER_NAME,
		 .id_table = pciidlist,
	},
#endif

	.name = DRIVER_NAME,
	.desc = DRIVER_DESC,
	.date = DRIVER_DATE,
	.major = DRIVER_MAJOR,
	.minor = DRIVER_MINOR,
	.patchlevel = DRIVER_PATCHLEVEL,
};

static struct pci_driver s3g_driver;
static int s3g_pcie_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
{
    s3g_card_t    *s3g = &s3g_cards[num_probed_card++];
    int           ret  = 0;

    s3g_memset(s3g, 0, sizeof(s3g_card_t));

#if DRM_VERSION_CODE > KERNEL_VERSION(3,10,52)
    driver.legacy_dev_list.prev = (struct list_head*)&driver.legacy_dev_list;
    driver.legacy_dev_list.next = (struct list_head*)&driver.legacy_dev_list;
    drm_get_pci_dev(pdev, ent, &driver);
#endif

    pci_set_drvdata(pdev, s3g);
    
#ifdef CONFIG_FB
    s3g->s3gfb_enable = s3g_fb;

    s3g_fb_detect_vesa(s3g, pdev);
#endif

    ret = pci_request_regions(pdev, s3g_driver.name);
    if(ret)
    {
        s3g_error("pci_request_regions() failed. ret:%x.\n", ret);
    }

    ret = pci_enable_device(pdev);
    if(ret)
    {
        s3g_error("pci_enable_device() failed. ret:%x.\n", ret);
    }

    pci_set_master(pdev);
   
    /*don't use the vga arbiter*/    
#if defined(CONFIG_VGA_ARB)
    vga_set_legacy_decoding(pdev, VGA_RSRC_NONE);
#endif

    ret = s3g_card_init(s3g, num_probed_card - 1, pdev);

    if(s3g_swfb)
    {
        swfb_probe(s3g, s3g_fb_mode);
    }
    else if(s3g->s3gfb_enable)
    {
#ifdef CONFIG_FB
        s3g_fb_probe(s3g, s3g_fb_mode, s3g_flip);
#endif
    }

    //krnl_skip_force_crt_on(s3g->adapter);

    return ret;
}

static void s3g_pcie_shutdown(struct pci_dev *pdev)
{
    s3g_card_t *s3g = pci_get_drvdata(pdev);

    if(s3g->s3gfb_enable)
    {
#ifdef CONFIG_FB
        s3g_fb_shutdown(s3g);
#endif
    }

}

static void __exit s3g_pcie_cleanup(struct pci_dev *pdev)
{
    s3g_card_t *s3g = pci_get_drvdata(pdev);
 
    if(s3g == NULL) return;

    if(s3g_swfb)
    {
        swfb_remove(s3g);
    }
    else if (s3g->s3gfb_enable)
    {
#ifdef CONFIG_FB
        s3g_fb_shutdown(s3g);
        s3g_fb_remove(s3g);
#endif
    }

    s3g_card_deinit(s3g);
 
    pci_set_drvdata(pdev, NULL);

    pci_release_regions(pdev);
}


static int s3g_pcie_suspend(struct pci_dev *pdev, pm_message_t state)
{
    s3g_card_t* s3g = pci_get_drvdata(pdev);

    s3g_info("s3g driver suspending, pm event = %d.\n", state.event);
#ifndef __mips__
#ifdef CONFIG_FB
    s3g_fb_set_pm(s3g, 1);
#endif

    krnl_save_state(s3g->adapter);

    /* disable IRQ */
    s3g_interrupt_deinit(s3g);

    pci_save_state(pdev);

    if (state.event & PM_EVENT_SLEEP)
    {
        pci_disable_device(pdev);
        pci_set_power_state(pdev, PCI_D3hot);
    }
#endif

    return 0;
}

static int s3g_pcie_resume(struct pci_dev *pdev)
{
    s3g_card_t* s3g = pci_get_drvdata(pdev);
    s3g_dpms_set_t dpms_set  = {0};
    unsigned int    uEDPSupport;
    int         ret = 0;

    s3g_info("s3g driver resume back.\n");

#ifndef __mips__
    pci_set_power_state(pdev, PCI_D0);

    pci_restore_state(pdev);

    if (pci_enable_device(pdev))
    {
        return -1;
    }
    pci_set_master(pdev);
   
    s3g_interrupt_reinit(s3g); 

    uEDPSupport =krnl_get_edpsupport(s3g->adapter);
    //If EDP support,we need force turn off.
    //SR register wolud be reset in cbPost
    //We need avoild EDP PowerSequence abnormal after S4 resume
    s3g_info("Ready to turn edp flow in s3g_pcie_resume\n");
    if((uEDPSupport & 0x1) ||(uEDPSupport & 0x2))
    {
        s3g_info("Turn OFF EDP in s3g_pcie_resume\n");
        dpms_set.output = UT_OUTPUT_DP5;
        dpms_set.mode   = 4; // force dpms off
        krnl_set_dpms(s3g->adapter, &dpms_set);
    }
 
    

    ret = krnl_restore_state(s3g->adapter);

    if(ret != 0)
    {
        return -1;
    }

#ifdef CONFIG_FB
    s3g_fb_set_pm(s3g, 0);
#endif
#endif

    return 0;
}


static struct pci_driver s3g_driver =
{
    .name = "s3g",
    .id_table = pciidlist,
    .probe    = s3g_pcie_probe,
#if LINUX_VERSION_CODE < 0x030800
    .remove   = __devexit_p(s3g_pcie_cleanup),
#else
    .remove   = __exit_p(s3g_pcie_cleanup),
#endif
    .shutdown = s3g_pcie_cleanup,
#ifndef KERNEL_2_4
    .suspend  = s3g_pcie_suspend,
    .resume   = s3g_pcie_resume
#endif
};


/*************************************** PCI functions ********************************************/

int s3g_get_command_status16(void *dev, unsigned short *command)
{
    return pci_read_config_word((struct pci_dev*)dev, 0x4, command);
}

int s3g_get_command_status32(void *dev, unsigned int *command)
{
    return pci_read_config_dword((struct pci_dev*)dev, 0x4, command);
}

int s3g_write_command_status16(void *dev, unsigned short command)
{
    return pci_write_config_word((struct pci_dev*)dev, 0x4, command);
}

int s3g_write_command_status32(void *dev, unsigned int command)
{
    return pci_write_config_dword((struct pci_dev*)dev, 0x4, command);
}


int s3g_get_bar1(void *dev, unsigned int *bar1)
{
    return pci_read_config_dword((struct pci_dev*)dev, 0x14, bar1);
}

int s3g_get_rom_save_addr(void *dev, unsigned int *romsave)
{
    return pci_read_config_dword((struct pci_dev*)dev, 0x30, romsave);
}

int s3g_write_rom_save_addr(void *dev, unsigned int romsave)
{
    return pci_write_config_dword((struct pci_dev*)dev, 0x30, romsave);
}

unsigned long s3g_get_rom_start_addr(void *dev)
{
    return pci_resource_start((struct pci_dev*)dev, 6);
}


void s3g_get_bus_config(void *dev, bus_config_t *bus)
{
    struct pci_dev *pdev = dev;

    pci_read_config_word(pdev, 0x2,  &bus->device_id);
    pci_read_config_word(pdev, 0x0,  &bus->vendor_id);
    pci_read_config_word(pdev, 0x4,  &bus->command);
    pci_read_config_word(pdev, 0x6,  &bus->status);
    pci_read_config_byte(pdev, 0x8,  &bus->revision_id);
    pci_read_config_byte(pdev, 0x9,  &bus->prog_if);
    
    pci_read_config_byte(pdev, 0xa,  &bus->sub_class);
    pci_read_config_byte(pdev, 0xb,  &bus->base_class);
    pci_read_config_byte(pdev, 0xc,  &bus->cache_line_size);
    pci_read_config_byte(pdev, 0xd,  &bus->latency_timer);
    pci_read_config_byte(pdev, 0xe,  &bus->header_type);
    pci_read_config_byte(pdev, 0xf,  &bus->bist);
    pci_read_config_word(pdev, 0x2c, &bus->sub_sys_vendor_id);    
    pci_read_config_word(pdev, 0x2e, &bus->sub_sys_id);
    pci_read_config_word(pdev, 0x52, &bus->link_status);

    bus->start_addr[0] = pci_resource_start(pdev, 0);
    bus->start_addr[1] = pci_resource_start(pdev, 1);
    bus->start_addr[2] = 0;
    bus->start_addr[3] = 0;
    bus->start_addr[4] = 0;
    bus->start_addr[5] = 0;


    bus->end_addr[0]   = pci_resource_end(pdev, 0);
    bus->end_addr[1]   = pci_resource_end(pdev, 1);
    bus->end_addr[2]   = 0;
    bus->end_addr[3]   = 0;
    bus->end_addr[4]   = 0;
    bus->end_addr[5]   = 0;

}


int s3g_init_bus_id(s3g_card_t *s3g, int card_num, void *dev)
{
    struct pci_dev *pdev = (struct pci_dev *)dev;
    int pci_domain = 0;
    int pci_bus    = pdev->bus->number;
    int pci_slot   = PCI_SLOT(pdev->devfn);
    int pci_func   = PCI_FUNC(pdev->devfn);

    s3g->busId = s3g_malloc(41);

    if(s3g->busId == NULL)
    {
        return ENOMEM;
    }

    s3g->len = snprintf(s3g->busId, 40, "pci:%04x:%02x:%02x.%d", pci_domain, pci_bus, pci_slot, pci_func);

    return 0;
}

#if LINUX_VERSION_CODE <= KERNEL_VERSION(3,10,52)
static struct pci_driver pcidriver = {
                    .name = "s3g_drm",
                    .id_table = pciidlist,
                 };
#endif

int s3g_init_driver(void)
{
    int ret;

    ret = pci_register_driver(&s3g_driver);
#if DRM_VERSION_CODE <= KERNEL_VERSION(3,10,52)
    ret = drm_pci_init(&driver, &pcidriver);
#endif
    return ret;
}

void s3g_deinit_driver(void)
{
#if DRM_VERSION_CODE <= KERNEL_VERSION(3,10,52)
    drm_pci_exit(&driver, &pcidriver);
#endif
    pci_unregister_driver(&s3g_driver);
}

int s3g_register_interrupt(s3g_card_t *s3g, void *isr)
{
    if(s3g->msi_enable)
    {
        s3g->msi_enable = pci_enable_msi(s3g->pdev) ? FALSE : TRUE;
        if(s3g->msi_enable)
        {
            krnl_enable_msi(s3g->adapter);
        }
    }

    return request_irq(((struct pci_dev*)s3g->pdev)->irq,
        (irqreturn_t (*)(int, void*))isr, IRQF_SHARED, "s3g", s3g);
}

void s3g_unregister_interrupt(s3g_card_t *s3g)
{
    free_irq(((struct pci_dev*)s3g->pdev)->irq, s3g);

    if(s3g->msi_enable)
    {
        pci_disable_msi((struct pci_dev*)s3g->pdev);
    } 
}
