/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#ifndef __s3g_def_h__
#define __s3g_def_h__

#define MAX_SCREENS         4
#define MAX_IGAS            2

#define SCREEN_SPLIT_MODE 0
#define USE_MODElINE_MODE 1

typedef unsigned int s3g_handle_t;
typedef unsigned int s3g_context_t;

#define ALIGN8 __attribute__ ((aligned(8)))

typedef unsigned long long s3g_ptr64_t;

typedef struct s3g_unique
{
    int unique_len;            /* in/out */
//    char __user* unique ;       /* out */
    s3g_ptr64_t unique ALIGN8;
}s3g_unique_t;

typedef struct _SPLIT_MODE_EXC
{
    unsigned int width;
    unsigned int height;
    unsigned int split_direction;
    unsigned int iga_applied;
    unsigned int bEnabled;
}SPLIT_MODE_EXC;

/*
    para 1,2(w/h)              : split mode width&&height;
    para  3(split_direction) : split screen to left/right or top/down, 0 means left/right, 1 means top/down;
    para  4(iga_applied)     : which iga apply to right or down side, 0 means iga1, 1 means iga2;
    para  5(bEnabled)        : if this split mode enabled or not, 0 means disabled, 1 means enabled;
*/
__attribute__((unused)) static  SPLIT_MODE_EXC Split_Mode_Table[] = 
{
    {3840, 1080, 0, 1, 1},
    {1920, 2160, 1, 1, 0},
    {2560, 1024, 0, 1, 0},
    {1280, 2048, 1, 1, 0},
};

typedef struct s3g_ctx
{
    s3g_context_t handle;
}s3g_ctx_t;

typedef struct s3g_clip_rect
{
     short x1;
     short y1;
     short x2;
     short y2;
}s3g_clip_rect_t;

typedef enum s3g_map_type
{
    S3G_FRAME_BUFFER = 0,
    S3G_REGISTERS = 1,
    S3G_SHM = 2,
}s3g_map_type_t;

typedef enum s3g_pwm_ctrl_type
{
    S3G_GET_FAN = 0,
    S3G_SET_FAN = 1,
    S3G_GET_BACKLIGHT = 2,
    S3G_SET_BACKLIGHT = 3,
}s3g_pwm_ctrl_type_t;

typedef struct s3g_lock
{
    int context;
}s3g_lock_t;

typedef struct s3g_clipnotify
{
    int x;
    int y;
    int width;
    int height;

    int numClipRects;
    int xId;
    unsigned int kdrawId;
    int scrnIndex;
    int iga_mask;
    unsigned int hAllocation;

    s3g_clip_rect_t* clipRects;
    /* How many clipRects knl malloc */
    int clipRectsVol;
}s3g_clipnotify_t;


typedef struct s3g_drawable
{
    int server_index;
    unsigned int xId;
    s3g_clipnotify_t clipNotify[MAX_SCREENS];
    int refCount;
}s3g_drawable_t;

typedef struct 
{
    unsigned int drawId;      /* in */
    int server_index;         /* out */
    int scrnIndex;            /* out */
    int fullscreen;           /* out */
    s3g_clip_rect_t rect;     /* out */
    int redirect;             /* out */
    int shadow;               /* out */
}s3g_get_drawsize_t;

typedef struct
{
    int  type;            /* in */
    int  monitor_type;    /* out */
    int  connected;       /* output */
    int  mode_redetected;

    int  edid_size;       /* output */
    void *edid;           /* output */
    
    int  mode_num;        /* output */
    int  mode_size;       /* output */
    void *modes;          /* output */

    int  _3dmode_num;     /* output */
    int  _3dmode_size;    /* output */
    void *_3dmodes;       /* output */
}s3g_output_caps_t;

typedef struct
{
    unsigned int    Size;
    /*For Display Port device*/
    unsigned int    LinkRate;             /* 6:1.62Gbps, 0xA: 2.7Gbps */
    unsigned int    LaneCount;            /* 1: one lane, 2: two lanes, 4: four lanes */
    unsigned int    TestPattern;          /* 0: No test pattern transmitted */
                                           /* 1: Color Ramps */
                                           /* 2: Black and white vertical lines */
                                           /* 3: color square */
    unsigned int    ClockSynAsyn;         /* 0: Link clock and stream clock asynchronous */
                                           /* 1: Link clock and stream clock synchronous*/
    unsigned int    DynamicRange;         /* 0: Vesa Range, 1: CEA range */
    unsigned int    ColorFormat;          /* 0: RGB, 1: YcbCR422, 2:YCbCr444 */
    unsigned int    YCbCrCoefficients;    /* 0: ITU601, 1: ITU709 */
    unsigned int    BitDepthPerComponet;  /* 0: 6 Bits */
                                           /* 1: 8 bits */
                                           /* 2: 10bits */
                                           /* 3: 12bits */
                                           /* 4: 16bits */
    /*Normal timing settings */                                           
    unsigned int    IsInterlaced;         /* 0: non-interlaced, 1: interlaced */
    unsigned int    HorTotal;             /* Horizontal total of transmitted video stream in pixel count */
    unsigned int    VerTotal;             /* Vertical total of transmitted video stream in line count */
    unsigned int    HorSyncStart;         /* Horizontal active start from Hsync start in pixel count */
    unsigned int    VerSyncStart;         /* Vertical active start from Vsync start in line count */
    unsigned int    HorSyncWidth;         /* HSync width in pixel count */
    unsigned int    HSyncPolarity;        /* 0: Negative, 1: Positive */ 
    unsigned int    VSyncPolarity;        /* 0: Negative, 1: Positive */
    unsigned int    VerSyncWidth;         /* VSync width in line count */
    unsigned int    HorWidth;             /* Active video width in pixel count */
    unsigned int    VerWidth;             /* Active video height in line count */
    unsigned int    DClk;                 /* DClk in Hz / 10000 */
    
}S3g_Custmized_Dest_Timing_t;

typedef struct 
{
    unsigned int Size;
    unsigned int Flag;                                /* 0: Use default method to select where timing come from*/
                                                   /* 1: Use user input customized timing, if this value is set, pUserCustDestTiming can not be CBIOS_NULL*/
                                                   /* 2: Use automated test DPCD timing*/
                                                   /* other value: reserved */
    S3g_Custmized_Dest_Timing_t User_Cust_Dest_Timing;   /* If Flag is set to 1, this structure is the user defined timing buffer*/
}S3g_Specify_Dst_Timing_t;

typedef struct
{
    int output;           /* in */
    int crtc;             /* in */
    int bpp;              /* in */

    int width;            /* in */
    int height;           /* in */
    int refresh;          /* in */

    int scale_width;      /* in */
    int scale_height;     /* in */

    int dst_width;        /* in */
    int dst_height;       /* in */
    int dst_refresh;      /* in */

    int interlace;        /* in */
    int scaler_type;      /* in */

    int is_3dmode;        /* in */
    int _3d_format;       /* in */
    S3g_Specify_Dst_Timing_t S3g_Specify_Dst_Timing;
}s3g_mode_info_t;

typedef struct
{
    int          num_crtcs;         /* out */
    int          num_outputs;       /* out */

    unsigned int outputs_supported; /* out */
    unsigned int outputs_connected; /* out */

}s3g_display_caps_t;

typedef struct
{
    int crtc;
    int is_crt;

    unsigned char *red, *green, *blue;
}s3g_gamma_info_t;

typedef struct
{
    int output;
    int mode;
}s3g_dpms_set_t;

typedef struct
{
    int output;
    s3g_pwm_ctrl_type_t type;
    int value;
}s3g_pwm_func_ctrl_t;

typedef enum
{
   S3G_CURSOR_SET_POS = 1,
   S3G_CURSOR_SHOW,
   S3G_CURSOR_HIDE,
}s3g_cursor_opcode;

typedef struct
{
    int     opcode;
    int     crtc;

    union
    {
        int fg;
        int x;
    };

    union
    {
        int bg;
        int y;
    };

    unsigned int hAllocation;
}s3g_set_cursor_t;

typedef struct
{
    unsigned int width;
    unsigned int height;
    unsigned int pitch;
    unsigned int bpp;

    unsigned int gbd;
    unsigned int hallocation;
    unsigned int offset;
}s3g_fb_info_t;

typedef struct  
{   
    int server_index;             /* in */
    int screen_index;             /* in */
    int index;                   /* in */
    int x, y;                    /* out */
    int width, height;           /* out */
    int refresh;                 /* out */
    int rotate;                  /* out */
} s3g_iga_info_t;

typedef struct  
{
    int server_index;                 /* in */
    int screen_index;                 /* in */

    int bXinerama;                   /* out */
    int bufBpp;                      /* out */
    int bDuoView;                    /* out */
    int bUseRxa;                     /* out */
    int iga2scrnIndex[MAX_IGAS];     /* out */
} s3g_display_info_t;


typedef struct
{
    // in variables
    int uDev;
    int xRes;
    int yRes;
    int refresh;
    int bInterlaced;
    int aspectRatio;

    // out variables
    int Clock;       /* pixel clock freq (kHz) */
    int HDisplay;    /* horizontal timing */
    int HBlankStart;
    int HSyncStart;
    int HSyncEnd;
    int HTotal;
    int VDisplay;    /* vertical timing */
    int VBlankStart;
    int VSyncStart;
    int VSyncEnd;
    int VTotal;
} s3g_mode_timing_info_t;

typedef struct
{
    char devicename[32];
    char busid[32];
    int  flag;
}s3g_device_info_t;

typedef struct 
{
    int server_index; /*in */
    int screen_index; /*in */

    int device_index; /*out, index of device(adapter)*/
    int sub_device_index; /*out, index of framebuffer device */	
}s3g_sub_device_info_t;

typedef struct
{
    unsigned int uDevCombin;      /* in */
    int bSimul;                   /* in */
    int bSupported;               /* out */
} s3g_check_device_combination_t;


typedef struct
{
    enum
    {
        S3G_INIT    = 0x01,
        S3G_CLEANUP = 0x02
    } func;                   /* in */

    int server_index;         /* in */
    int xinerama;             /* in */

    unsigned int device;      /* output */
} s3g_2d_init_t;



typedef struct
{
    unsigned int   len;  /* in */
    unsigned int   hSrcAllocation;   /* in */
    unsigned int   hDstAllocation;   /* in */
    unsigned int   hMaskAllocation;  /* in */
    unsigned int   kernel_mode_command;
    unsigned char *cmd;  /* in */
}s3g_2d_draw_t;



typedef struct
{
    int server_index;
    int scrnIndex;
    int master;
    int samm;
    int mamm;
    int x;
    int y;
    int width;
    int height;
    int bpp;
    int pitch;
    unsigned int device;
    unsigned int hwFormat;
    int          rotate;
    int          gbd;
    int          use_rxa;
    int          bDuoView;
    unsigned long fbOffset;
    unsigned int hAllocation;
}s3g_2d_screen_t;

typedef struct
{
    unsigned int width;
    unsigned int height;
    unsigned int bpp;
    unsigned int pitch;
    int          rotate;
    int          gbd;
    int          tiled;
    unsigned long offset;
    unsigned int allocation;
    unsigned int size;
    unsigned int segment_id;
}s3g_2d_create_allocation_t;

typedef struct
{
    unsigned int        hDevice;
    unsigned int        iga_index;     /* in */
    void                *private_data; /* in/out */
    unsigned int        private_size;  /* in */
    void                *handle;       /* out */
}s3g_create_overlay_t;

typedef struct
{
    unsigned int        hDevice; /* in */
    void                *handle; /* in */
}s3g_destroy_overlay_t;


typedef struct
{
    unsigned int       hDevice; /* in */
    void               *handle;       /* in */
    void               *private_data; /* in/out */
    unsigned int       private_size;  /* in */
    int                server_index;  /* in */
    int                screen_index;  /* in */
}s3g_update_overlay_t;

typedef struct
{
    unsigned int       hDevice; /* in */
    void               *handle;       /* in */
    unsigned int       mode_changed;  /* out */
}s3g_flip_overlay_t;

typedef struct
{
    unsigned int       hDevice;      /* in */
    int                server_index; /* in */
    int                screen_index; /* in */ 
    int                delay_num;    /* in */ 
    int                fps;          /* in */ 
    int                check;        /* in */ 
}s3g_set_overlay_info_t;

typedef struct
{
    unsigned int       hDevice;      /* in */
    unsigned int       crtc;         /* in */
    unsigned int       hOverlay;     /* out */
}s3g_acquire_overlay_t;

typedef struct
{
    unsigned int       hDevice;      /* in */
    unsigned int       hOverlay;     /* in */
}s3g_release_overlay_t;

typedef struct _s3g_overlay_flag
{
    union
    {
        struct
        {
            unsigned int dst_color_key       : 1;
            unsigned int dst_color_ket_range : 1;
            unsigned int src_color_key       : 1;
            unsigned int src_color_key_range : 1;
            unsigned int constant_alpha      : 1;
            unsigned int ps_alpha            : 1;
            unsigned int ss_alpha            : 1;
            unsigned int reserved            :25;
        };
        unsigned int value;
    };
} s3g_overlay_flag_t;

typedef struct
{
    unsigned int       hOverlay;

    unsigned int       crtc;
    s3g_clip_rect_t    src_rect;
    s3g_clip_rect_t    dst_rect;

    unsigned int       surface;

    s3g_overlay_flag_t flag;

    unsigned int       src_colorkey;
    unsigned int       dst_colorkey;

    unsigned int       alpha_blend;

    unsigned int       cc1, cc2, cc3;

    unsigned int       rotate;
}s3g_set_overlay_t;

typedef struct
{
    unsigned int    hOverlay;
    unsigned int    crtc;
    unsigned int    surface;
}s3g_overlay_flip_t;

typedef enum
{
    S3G_FRC_ENABLE = 1,
    S3G_FRC_DISABLE,
    S3G_FRC_UPDATE_PATTERN_TABLE,
}s3g_frc_opcode;

typedef struct
{
    int        opcode;
    int        crtc;

    int        pattern_table_size;
    void       *pattern_table;
}s3g_set_frc_t;

typedef enum 
{
    S3G_CEC_INDEX1 = 0,
    S3G_CEC_INDEX2,
    S3G_CEC_INDEX_COUNT
}s3g_cec_index;

typedef struct 
{
    unsigned int       Size;           // Size of CBIOS_CEC_TRANSIMIT_MESSAGE_PARA
    s3g_cec_index    CECIndex;       // CEC index
    unsigned char     DestAddr;       // CEC Initiator command destination address
    unsigned int       CmdLen;         // CEC Initiator command length. The valid value is [0:16]
    unsigned char     Command[16];    // Initiator Command sent by CEC
    signed int           bBroadcast;     // = TRUE: broadcast; = FALSE: direct access  
    unsigned char      RetryCnt;       // CEC Initiator Retry times. Valid values must be 1 to 5.
}s3g_transmit_msg_t;

typedef struct 
{
    unsigned int       Size;            // Size of CBIOS_CEC_RECEIVE_MESSAGE_PARA
    s3g_cec_index    CECIndex;       // CEC index
    unsigned char     SourceAddr;     // CEC Follower received source address
    unsigned int       CmdLen;         // CEC Follower command length. The valid value is [0:16]
    unsigned char     Command[16];    // Follower received command
    signed int           bBroadcast;     // = TRUE: broadcast; = FALSE: direct access  
}s3g_receive_msg_t;

typedef enum
{
    S3G_CEC_MONITOR_CTL = 3,
    S3G_CEC_HOST_CTL
}s3g_cec_opcode;

typedef enum
{
    S3G_CEC_Get_PHY_ADDR = 0 ,
    S3G_CEC_Set_LA_ADDR,
    S3G_CEC_Get_LA_ADDR
}s3g_cec_host_opcode;

typedef struct 
{
    unsigned char          LogicalAddr;
    unsigned int          PhysicalAddr;
}s3g_cec_addr;

typedef struct
{
    int crtc;              /* input */
    int output;            /* input */
    int monitor_off;       /* input */
    s3g_transmit_msg_t tx_cec;
    s3g_receive_msg_t rx_cec;
    /* add more when neeed */
}s3g_cec_monitor_ctl_t;

typedef struct
{
    int host_off;                  /* input */
    s3g_cec_host_opcode OpCoode;
    s3g_cec_addr        CECAddr;
    s3g_cec_index       CECIndex; 
    /* add more when neeed */
}s3g_cec_host_ctl_t;

typedef struct
{
    s3g_cec_opcode            opcode;
    s3g_cec_monitor_ctl_t     monitor_ctl;
    s3g_cec_host_ctl_t        host_ctl;
}s3g_cec_ctl_t;

typedef enum
{
    S3G_ROTATE_NONE = 0,
    S3G_ROTATE_90_DEGREE,
    S3G_ROTATE_180_DEGREE,
    S3G_ROTATE_270_DEGREE,

}s3g_rotate_type_t;

typedef struct
{
    int server_index;  /* in */
    int screen_index;  /* in */
    int crtc;
    int x, y;

    unsigned int rotate_type;
    unsigned int surface;
    unsigned int shadow_surface; /* if shadow surface set means rotate by sw */
}s3g_ps_info_t;

typedef struct
{
    unsigned int hContext;
    unsigned int crtc_mask;
    unsigned int flip_interval;
    unsigned int left_surface;
    unsigned int right_surface;
}s3g_flip_info_t;

typedef struct
{
    unsigned char engine_mask;  /* in */
    unsigned int  hAllocation;  /* in */
}s3g_wait_allocation_idle_t;

typedef struct
{
    int server_index;  /* in */
    int screen_index;  /* in */

    int input_size;    /* in */
    int output_size;   /* in */

    s3g_ptr64_t input  ALIGN8;/* in */
    s3g_ptr64_t output ALIGN8;/* output */
}s3g_escape_call_t;

typedef struct
{
    unsigned long       mmioPhyAddr;  /* output */
}s3g_get_address_config_t;

typedef enum  
{
    S3G_QUERY_VBIOS_VERSION,
    S3G_QUERY_TOTAL_VRAM_SIZE,
    S3G_QUERY_CPU_VISIBLE_VRAM_SIZE,
    S3G_QUERY_RESERV_VRAM_SIZE,
    S3G_QUERY_CHIP_ID,
    S3G_QUERY_CRTC_OUTPUT,
    S3G_QUERY_INFO_MAX,
    S3G_QUERY_MONITOR_ATTRIBUTE,
    S3G_QUERY_ENGINE_CLOCKS,
    S3G_QUERY_GPU_CLOCKS,
    S3G_QUERY_MEMORY_CLOCKS,
}S3G_QUERY_INFO_TYPE;

typedef struct
{
    unsigned int type;    /* in */
    unsigned int sel_device; /* in */
    union
    {
        unsigned int value;    /* out */
        unsigned int output[MAX_IGAS]; /* out */
    };
    short   MonitorHorSize; /* out */
    short   MonitorVerSize; /* out */
}s3g_query_info_t;

typedef struct _s3g_demux_buf_gpu_addr
{
    unsigned int vbb_start;
    unsigned int vbb_end;
    unsigned int abb_start;
    unsigned int abb_end;
    unsigned int vib_start;
    unsigned int vib_end;
    unsigned int sbb_start;
    unsigned int sbb_end;
}s3g_demux_buf_gpu_addr_t;

typedef struct _s3g_demux_buf_cpu_addr
{
    void *vbb_start;
    void *vbb_end;
    void *abb_start;
    void *abb_end;
    void *vib_start;
    void *vib_end;
    void *sbb_start;
    void *sbb_end;
}s3g_demux_buf_cpu_addr_t;

typedef struct _s3g_demux_info
{
    s3g_demux_buf_gpu_addr_t gpu_virt;
    s3g_demux_buf_cpu_addr_t umd_cpu_virt;
    s3g_demux_buf_cpu_addr_t kmd_cpu_virt;
}s3g_demux_info_t;

typedef enum
{
    S3G_DEMUX_OP_MAP   = 1,
    S3G_DEMUX_OP_UNMAP,
    S3G_DEMUX_OP_INIT,
    S3G_DEMUX_OP_DEINIT
}S3G_DEMUX_OP_TYPE;

typedef struct _s3g_demux_op
{
    unsigned int        op;
    s3g_demux_info_t    demux_info;
}s3g_demux_op_t;

typedef struct
{
    unsigned int       enabled;  /* input */
}s3g_set_shrink_t;

typedef struct
{
    unsigned int       enabled;  /* output */
}s3g_get_shrink_t;

#define  S3G_EVENT_VSYNC_0_INTERRUPT   (1 << 0)
#define  S3G_EVENT_VSYNC_1_INTERRUPT   (1 << 1)
#define  S3G_EVENT_HOTPLUG             (1 << 4)


#define  S3G_EVENT_ALL  (S3G_EVENT_VSYNC_0_INTERRUPT | S3G_EVENT_VSYNC_1_INTERRUPT | S3G_EVENT_HOTPLUG)

#define  S3G_WAIT_ADD        1
#define  S3G_WAIT_REMOVE     2
#define  S3G_WAIT_EVENT      3
#define  S3G_WAIT_GET_EVENT  4

typedef struct
{
    unsigned int opcode;        /* in */
    unsigned int events;        /* in/out */
    unsigned int timeout;
}s3g_wait_func_t;

#define S3G_NAME "s3g"
#define S3G_LENOVO_DISPLAY_ORDER_PATCH 1

/* define the device type  */
#define UT_OUTPUT_NONE      0x00 /* No device. */
/* ubuntu12.04 unity+xrandr have sync problem when connected output are CRT1+LCD with clone mode, here switch our internal output define to avoid this problem. */
#define UT_OUTPUT_LCD       0x01 /* LCD *//*In fact it should be called pannel*/
#if S3G_LENOVO_DISPLAY_ORDER_PATCH
#define UT_OUTPUT_CRT1      0x10000 /* CRT1 */
#else
#define UT_OUTPUT_CRT1      0x02 /* CRT1 */
#endif
#define UT_OUTPUT_TV        0x04 /* TV */
#define UT_OUTPUT_DVO       0x08 /* DVI */
#define UT_OUTPUT_CRT2      0x10 /* CRT2 */
#define UT_OUTPUT_HDTV      0x20 /* HDTV */
#define UT_OUTPUT_DVI2      0x40 /* DVI2 */

/* directly use the format of cbios for the high 8 bits */
#define UT_OUTPUT_LCD2      0x100 /*LCD2*/
#define UT_OUTPUT_HDMI3     0x200 /*HDMI(3)*/
#define UT_OUTPUT_DVI3      0x400 /*DVI3*/
#define UT_OUTPUT_DVI4      0x800 /*DVI4*/
#define UT_OUTPUT_HDMI1     0x1000 /*HDMI1*/
#define UT_OUTPUT_HDMI2     0x2000 /*HDMI2*/
#define UT_OUTPUT_HDMI4     0x4000 /*HDMI4*/
#if S3G_LENOVO_DISPLAY_ORDER_PATCH
#define UT_OUTPUT_DP5       0x02 /*DP5*/ 
#define UT_OUTPUT_DP6       0x8000 /*DP6*/
#else
#define UT_OUTPUT_DP5       0x8000 /*DP5*/ 
#define UT_OUTPUT_DP6       0x10000 /*DP6*/
#endif
#define UT_OUTPUT_LCD3      0x20000  /*LCD3*/

#define UT_OUTPUT_MASK 0x3FF7F
#define UT_OUTPUT_CRTS ( UT_OUTPUT_CRT1 | UT_OUTPUT_CRT2 )
#define UT_OUTPUT_LCDS ( UT_OUTPUT_LCD |UT_OUTPUT_LCD2 )
#define UT_OUTPUT_DVIS ( UT_OUTPUT_DVO |UT_OUTPUT_DVI2 |UT_OUTPUT_DVI3 | UT_OUTPUT_DVI4 )
#define UT_OUTPUT_TVS  ( UT_OUTPUT_TV | UT_OUTPUT_TV2 )
#define UT_OUTPUT_HDMIS ( UT_OUTPUT_HDMI1 | UT_OUTPUT_HDMI2 | UT_OUTPUT_HDMI3 | UT_OUTPUT_HDMI4  )
#define UT_OUTPUT_DPS  ( UT_OUTPUT_DP5 | UT_OUTPUT_DP6)

static inline char* ut_strncpy(char* dst, const char* src, int len)
{
    char * d = dst;
    while(len)
    {
        len--;
        if((*d++ = *src++) == '\0')
             break;
    }

    while(len--)
        *d++ = '\0';

    return dst;
}

static inline char * ut_dev_str(int device)
{
    static char string[64];
    char *p=&string[0];

    if (device & UT_OUTPUT_NONE)
        return 0;

    if (device & UT_OUTPUT_LCD) 
    {
        ut_strncpy(p,"LCD+",4);
        p += 4;
    }
    
    if (device & UT_OUTPUT_CRT1) 
    {   
        ut_strncpy(p,"CRT1+",5);
        p += 5;
    }

    if (device & UT_OUTPUT_TV)
    {
        ut_strncpy(p,"TV+",3);
        p += 3;
    }
    if (device & UT_OUTPUT_DVO) 
    {
        ut_strncpy(p,"DVO+",4);
        p += 4;
    }
    if (device & UT_OUTPUT_DVI2)
    {
        ut_strncpy(p,"DVI2+",5);
        p += 5;
    }
    if (device & UT_OUTPUT_CRT2) 
    {
        ut_strncpy(p,"CRT2+",5);
        p += 5;
    }
    if (device & UT_OUTPUT_HDTV)
    {
        ut_strncpy(p,"HDTV+",5);
        p += 5;
    }

    if (device & UT_OUTPUT_LCD2)
    {
        ut_strncpy(p,"LCD2+",5);
        p += 5;
    }

    if (device & UT_OUTPUT_HDMI3)
    {
        ut_strncpy(p,"HDMI3+",6);
        p += 6;
    }
    if (device & UT_OUTPUT_DVI3) 
    {
        ut_strncpy(p,"DVI3+",5);
        p += 5;
    }
    if (device & UT_OUTPUT_DVI4) 
    {
        ut_strncpy(p,"DVI4+",5);
        p += 5;
    }
    if (device & UT_OUTPUT_HDMI1) 
    {
        ut_strncpy(p,"HDMI1+",6);
        p += 6;
    }
    if (device & UT_OUTPUT_HDMI2)
    {
        ut_strncpy(p,"HDMI2+",6);
        p += 6;
    }
    if (device & UT_OUTPUT_HDMI4) 
    {
        ut_strncpy(p,"HDMI4+",6);
        p += 6;
    }
    if (device & UT_OUTPUT_DP5)
    {
        ut_strncpy(p,"DP5+",4);
        p += 4;
    }
    if (device & UT_OUTPUT_DP6) 
    {
        ut_strncpy(p,"DP6+",4);
        p += 4;
    }
    if (device & UT_OUTPUT_LCD3)
    {
        ut_strncpy(p,"LCD3+",5);
        p += 5;
    }
    /* delete the last '+' */
    if(p == string)
    {
       *p = '\0';
    }
    else
    {
       *(p-1) = '\0';
    }

    return string;
}

static inline s3g_ptr64_t ptr_to_ptr64(void *ptr)
{
    return (s3g_ptr64_t)(unsigned long)ptr;
}

static inline void *ptr64_to_ptr(s3g_ptr64_t ptr64)
{
    return (void*)(unsigned long)ptr64;
}

#endif

