/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#ifndef __S3G_DRIVER_H
#define __S3G_DRIVER_H

#include "s3.h"
#include "s3g_def.h"
#include "s3g_types.h"
#include "os_interface.h"


#define S3G_MAX_CARDS         16


typedef struct s3g_file s3g_file_t;

typedef int (*s3g_ioctl_t)(s3g_file_t* priv, unsigned int cmd, unsigned long arg);

typedef int (*s3g_ioctl_t_2)(s3g_file_t* priv, unsigned int cmd, unsigned long arg, void *);


typedef int (*irq_func_t)(void*);

typedef struct
{
    void              *adapter;
#ifdef S3G_PCIE_BUS
    struct pci_dev    *pdev;
#else
    struct platform_device *pdev;
#endif
    char              *busId;
    int               len;
    int               index;

    struct os_mutex   *lock;

    int               msi_enable;
    unsigned int      freezable_patch;      /* Freezable patch for COS. */

    irq_func_t        isr_func;
    void              *isr_bf_func;

    struct tasklet_struct tasklet;

    wait_queue_head_t wait_queue;

    spinlock_t        wait_list_lock;
    struct list_head  wait_list;

    int               enter_vt;    /* if enter vt means leave console mode */
    unsigned int      s3gfb_pending_events;
    int               s3gfb_enable;
    int               vga16fb_enable;
    int               vesafb_enable;
    int               reserved_vmem;
    s3g_mode_info_t   vesafb_mode;

    struct fb_info    *info;
    struct led_classdev lcd;
}s3g_card_t;

struct s3g_file
{
    struct file       *filp;
    s3g_card_t        *card;

    void              *map;
    unsigned int      gpu_device;

    int               hold_lock;
    int               freezable;

    unsigned int      curr_events;
    unsigned int      wait_events;
    struct list_head  wait_list;
    wait_queue_head_t wait_queue;
};

extern s3g_ioctl_t s3g_ioctls[];

extern int  s3g_card_init(s3g_card_t *s3g, int card_num, void *pdev);
extern int  s3g_card_deinit(s3g_card_t *s3g);

extern void s3g_interrupt_init(s3g_card_t *s3g);
extern void s3g_interrupt_reinit(s3g_card_t *s3g);
extern void s3g_interrupt_deinit(s3g_card_t *s3g);

extern int  s3g_map_system_io(struct vm_area_struct *vma, s3g_map_argu_t *map);
extern int  s3g_map_system_ram(struct vm_area_struct *vma, s3g_map_argu_t *map);

extern int  s3g_add_wait(s3g_card_t* s3g, s3g_file_t *priv, unsigned int);
extern int  s3g_remove_wait(s3g_card_t* s3g, s3g_file_t* priv);
extern unsigned int s3g_get_event(s3g_card_t* s3g, s3g_file_t* priv);
extern int  s3g_post_event(s3g_card_t *s3g, unsigned int events);

extern void  s3g_init_2d_screen(s3g_card_t *card, s3g_2d_screen_t *screen);
extern void  s3g_deinit_2d_screen(s3g_card_t *card, s3g_2d_screen_t *screen);

extern int   s3g_fb_probe(s3g_card_t *s3g, char *mode_string, int flip);
extern int   s3g_fb_shutdown(s3g_card_t *s3g);
extern int   s3g_fb_remove(s3g_card_t *s3g);
extern void  s3g_fb_set_pm(s3g_card_t *s3g, int bsuspend);
extern void  s3g_fb_detect_vesa(s3g_card_t *s3g, void *pdev);

extern int   swfb_probe(s3g_card_t *card, char *mode_string);
extern int   swfb_remove(s3g_card_t *card);

#ifdef NEW_S3GFB
extern int   s3gfb_probe(s3g_card_t *s3g, char *mode_string, int flip);
extern int   s3gfb_shutdown(s3g_card_t *s3g);
extern int   s3gfb_remove(s3g_card_t *s3g);
extern void  s3gfb_set_pm(s3g_card_t *s3g, int bsuspend);
extern void  s3gfb_detect_vesa(s3g_card_t *s3g, void *pdev);
extern int   s3gfb_handle_event(s3g_card_t *s3g, unsigned int events);

#define   s3g_fb_probe          s3gfb_probe
#define   s3g_fb_shutdown       s3gfb_shutdown
#define   s3g_fb_remove         s3gfb_remove
#define   s3g_fb_set_pm         s3gfb_set_pm
#define   s3g_fb_detect_vesa    s3gfb_detect_vesa
#define   swfb_probe
#define   swfb_remove

#else

extern int   s3g_fb_probe(s3g_card_t *s3g, char *mode_string, int flip);
extern int   s3g_fb_shutdown(s3g_card_t *s3g);
extern int   s3g_fb_remove(s3g_card_t *s3g);
extern void  s3g_fb_set_pm(s3g_card_t *s3g, int bsuspend);
extern void  s3g_fb_detect_vesa(s3g_card_t *s3g, void *pdev);

extern int   swfb_probe(s3g_card_t *card, char *mode_string);
extern int   swfb_remove(s3g_card_t *card);
#endif

extern void* krnl_init_adapter(void *pdev, int reserved_vmem, int index);
extern void  krnl_deinit_adapter(void *data);
extern void  krnl_notify_vesa_mode(void *data, s3g_mode_info_t *vesa_mode);
extern void  krnl_final_cleanup(void *data, unsigned int gpu_device);
extern int   krnl_get_isr_func(void *adapter, void **isr_func, void **isr_bf_func);
extern void  krnl_enable_msi(void *data);
extern void  krnl_save_state(void *adapter);
extern int   krnl_restore_state(void *adapter);
extern void  krnl_get_pgtbl_entry_dynamic(void *adapter, unsigned long phys_addr, unsigned long size, struct os_pages_memory *pages);

extern void  krnl_query_display_caps(void *data, s3g_display_caps_t *display_caps);
extern int   krnl_check_device_combination(void *data, s3g_check_device_combination_t *device_combination);
extern void  krnl_set_mode(void *data, s3g_mode_info_t *mode_set);
extern void  krnl_set_cursor(void *data, s3g_set_cursor_t *cursor_set);
extern void  krnl_set_dpms(void *data, s3g_dpms_set_t *dpms_set);
extern void krnl_set_vcpflag(void *data, int setvcpflag);
extern void krnl_set_boot_noise_patch_end(void *data, int status);

extern unsigned int krnl_get_edpsupport(void *data);
extern unsigned int krnl_create_2ddevice(void *data, void *process_context);
extern int   krnl_destroy_2ddevice(void *data, void *process_context);

extern void  krnl_destroy_2d_allocation(void *data, unsigned int handle);
extern void  krnl_kickoff_2d_draw(void *data, s3g_2d_draw_t *draw_2d);
extern void  krnl_wait_chip_idle(void *data);
extern void  krnl_query_output_caps(void *data, s3g_output_caps_t *output_caps, int from_user_space);
extern void  krnl_set_ps(void *data, s3g_ps_info_t *ps);
extern void  krnl_fill_allocation(void *data, s3g_fill_allocation_t *s3g_fill_data);
extern int   krnl_create_allocation(void* data, s3g_create_allocation_t *create_data);

extern int   krnl_disable_interrupt(void *data);

extern int   krnl_query_info(void* data, s3g_query_info_t *info);

extern int   krnl_pwm_func_ctrl(void *data, s3g_pwm_func_ctrl_t *pwm_func_ctrl);

#define OS_CALLBACK_POST_EVENT     1

extern int krnl_set_callback_func(void *data, int type, void *func, void *argu);

extern int s3g_init_bus_id(s3g_card_t *s3g, int card_num, void *dev);
extern int s3g_init_driver(void);
extern void s3g_deinit_driver(void);
extern int s3g_register_interrupt(s3g_card_t *s3g, void *isr);
extern void s3g_unregister_interrupt(s3g_card_t *s3g);

/*
 * export for video capture driver
 */
extern unsigned int krnl_create_allocation_export(void *data, s3g_2d_create_allocation_t *create_data);
extern void krnl_destroy_allocation_export(void *data, unsigned int handle);
extern void *krnl_lock_allocation_export(void *data, unsigned int handle, unsigned int flags);
extern void krnl_unlock_allocation_export(void *data, unsigned int handle);

extern unsigned int s3g_create_allocation_export(s3g_2d_create_allocation_t *create_data);
extern void s3g_destroy_allocation_export(unsigned int handle);
extern void *s3g_lock_allocation_export(unsigned int handle, unsigned int flags);
extern void s3g_unlock_allocation_export(unsigned int handle);
extern void s3g_kickoff_dma_export(s3g_2d_draw_t *s3g_2d_draw);

extern void* s3g_get_global_adapter(void);
extern void krnl_skip_force_crt_on(void *data);

extern void krnl_power_off_crtc(void *data);

#endif

