/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#ifndef __S3G_FB_H__
#define __S3G_FB_H__

enum
{
    CHIP_CMODEL,
    CHIP_CLB,
    CHIP_DST,
    CHIP_CSR,
    CHIP_INV,
    CHIP_H5,
    CHIP_H5S1,
    CHIP_H6S2,
    CHIP_CMS,
    CHIP_METRO,
    CHIP_MANHATTAN,
    CHIP_MATRIX,
    CHIP_DST2,
    CHIP_DST3,
    CHIP_DUMA,
    CHIP_H6S1,
    CHIP_DST4,
    CHIP_EXC1,      //Excalibur-1
    CHIP_E2UMA,     //E2UMA
    CHIP_ELT,       //Elite
    CHIP_ELT1K,
    CHIP_LAST,
};

enum
{
    FB_OPS_FILLRECT,
    FB_OPS_COPYAREA,
    FB_OPS_IMAGEBLIT,
    FB_OPS_SYNC,
};

typedef struct
{
    unsigned int x;
    unsigned int y;
    unsigned int width;
    unsigned int pitch;
    unsigned int height;
    unsigned int bpp;

    unsigned int gbd;
    unsigned int hAllocation;
    unsigned int fbOffset;
    void         *fbVirtual;

    int pseudo_palette[17];
} s3g_fb_screen_t;

typedef struct
{
    unsigned int hAllocation;
    unsigned int fbOffset;
    int gbd;
    int dx;
    int dy;
    int width;
    int height;
    int color;
    int rop;
} s3g_fillrect_t;

typedef struct
{
    unsigned int hAllocation;
    unsigned int fbOffset;
    unsigned int pitch;
    int gbd;
    int dx;
    int dy;
    int width;
    int height;
    int sx;
    int sy;
} s3g_copyarea_t;

typedef struct
{
    unsigned int hAllocation;
    unsigned int fbOffset;
    int gbd;
    int dx;
    int dy;
    int width;
    int height;
    int fg_color;
    int bg_color;
    int depth;
    const char* data;
} s3g_image_t;

typedef struct
{
    unsigned int hAllocation;
    unsigned int xoffset;
    unsigned int yoffset;
    unsigned int pitch;
    unsigned int offset;
    unsigned int gbd;
    unsigned int bpp;
    int pipe;
} s3g_fb_set_ps_t;

typedef struct
{
    void (*fillrect)(void* adapter, const s3g_fillrect_t* s3g_rect);
    void (*copyarea)(void* adapter, const s3g_copyarea_t* s3g_area);
    void (*imageblit)(void* adapter, const s3g_image_t* s3g_image);
    void (*sync)(void* adapter);
    int  (*getheightalign)(int bit_count);
    void (*set_ps)(void* adapter, s3g_fb_set_ps_t *set_ps);
    int  (*get_local_mem_type)(void);
} s3g_fb_chip_func_t;

typedef struct
{
    int width;
    int height;
    int bpp;
    int refresh;
    int dstWidth;
    int dstHeight;
    int native_mode;
    int flag;
} s3g_fb_mode_t;

typedef struct
{
    int id;

    s3g_mode_info_t mode_info;
    s3g_ps_info_t ps_info;
} s3g_fb_crtc_t;


typedef struct
{
    int id;
    int type;

    int connected;
    s3g_fb_crtc_t *crtc;
    
    int max_width;
    int max_height;

    int mode_num;
    s3g_fb_mode_t* mode_list;
    s3g_fb_mode_t* native_mode;
    char* edid;
} s3g_fb_output_t;


typedef struct
{
    int crtc_count;
    int output_count;

    int dev_supported;
    int dev_connected;
    int dev_preferred[2];

    s3g_fb_crtc_t* crtc;
    s3g_fb_output_t* output;
} s3g_fb_device_t;

typedef struct
{
    void* adapter;

    int   disabled;

    s3g_fb_device_t device;

    unsigned int hDevice;
    unsigned int hCursorAllocation;
    void         *cursorVirtual;
    unsigned int hPrivCursorAllocation;
    void         *privCursorVirtual;

    unsigned long fbBase;

    s3g_fb_chip_func_t* chip_func;
} s3g_fb_par_t;

extern s3g_fb_chip_func_t s3g_fb_function_dst;
extern s3g_fb_chip_func_t s3g_fb_function_elt;
extern s3g_fb_chip_func_t s3g_fb_function_exc;

extern void s3g_fbi_parse_screen_mode(void *adapter, char *mode_string, s3g_fb_mode_t *default_mode);
extern int  s3g_fbi_init_device(void *adapter, s3g_fb_device_t *device);
extern void s3g_fbi_init_crtc_mode_set(void *adapter, s3g_fb_device_t *device, s3g_fb_mode_t *mode);
extern int  s3g_fbi_fix_screen_mode(void* adapter, s3g_fb_device_t* device, s3g_fb_mode_t* default_mode);
extern int s3g_fbi_get_moniter_physical_size(void* adapter, s3g_fb_device_t *device, s3g_fb_mode_t *prefer_mode, s3g_query_info_t *query);
extern void s3g_fbi_deinit_device(void *adapter, s3g_fb_device_t *device);
extern void s3g_fbi_get_dev_preferred(void *adapter, int dev_connected, int *preferred);
extern int  s3g_fbi_mode_valid(s3g_fb_output_t *device, s3g_mode_info_t *mode_set);
extern void *s3g_fbi_map(s3g_fb_par_t *s3g_par, unsigned int hAllocation);
extern void s3g_fbi_unmap(s3g_fb_par_t *s3g_par, unsigned int hAllocation);

#endif



