/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#ifndef __S3G_IOCTL_H__
#define __S3G_IOCTL_H__

#if defined(__linux__)
#include <asm/ioctl.h>
#define S3G_IOCTL_NR(n)         _IOC_NR(n)
#define S3G_IOC_VOID            _IOC_NONE
#define S3G_IOC_READ            _IOC_READ
#define S3G_IOC_WRITE           _IOC_WRITE
#define S3G_IOC_READWRITE       _IOC_READ | _IOC_WRITE
#define S3G_IOC(dir, group, nr, size) _IOC(dir, group, nr, size)
#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
#if defined(__FreeBSD__) && defined(IN_MODULE)
#undef ioctl
#include <sys/ioccom.h>
#define ioctl(a,b,c)            xf86ioctl(a,b,c)
#else
#include <sys/ioccom.h>
#endif
#define S3G_IOCTL_NR(n)         ((n) & 0xff)
#define S3G_IOC_VOID            IOC_VOID
#define S3G_IOC_READ            IOC_OUT
#define S3G_IOC_WRITE           IOC_IN
#define S3G_IOC_READWRITE       IOC_INOUT
#define S3G_IOC(dir, group, nr, size) _IOC(dir, group, nr, size)
#endif

#ifndef __user
#define __user
#endif

#define S3G_MAJOR       227
#define S3G_MAX_MINOR   255


enum s3g_ioctl_nr
{
    ioctl_nr_getbusid,
    ioctl_nr_get_displayinfo,
    ioctl_nr_get_igainfo,  
    ioctl_nr_addctx,
    ioctl_nr_rmctx,
    ioctl_nr_lock,
    ioctl_nr_unlock,
    ioctl_nr_clipnotify,
    ioctl_nr_adddraw,
    ioctl_nr_rmdraw,
    ioctl_nr_getdrawsize,
    ioctl_nr_query_display_caps,
    ioctl_nr_query_output_caps,
    ioctl_nr_mode_set,
    ioctl_nr_cursor,
    ioctl_nr_gamma,
    ioctl_nr_dpms,
    ioctl_nr_get_mode_timing,
    ioctl_nr_leave_vt,
    ioctl_nr_enter_vt,
    ioctl_nr_check_device_combination,
    ioctl_nr_2d_init,
    ioctl_nr_wait_chip_idle,
    ioctl_nr_escape_call,
    ioctl_nr_2d_draw,
    ioctl_nr_init_screen,
    ioctl_nr_deinit_screen,
    ioctl_nr_acquire_aperture,
    ioctl_nr_release_aperture,
    ioctl_nr_set_ps,
    ioctl_nr_enum_sub_device,
    ioctl_nr_create_allocation,
    ioctl_nr_destroy_allocation,
    ioctl_nr_lock_allocation,
    ioctl_nr_unlock_allocation,
    ioctl_nr_wait_allocation_idle,
    ioctl_nr_create_overlay,
    ioctl_nr_destroy_overlay,
    ioctl_nr_update_overlay,
    ioctl_nr_flip_overlay,
    ioctl_nr_set_overlay_info,
    ioctl_nr_video_get_hw_caps,
    ioctl_nr_get_address_config,
    ioctl_nr_query_info,
    ioctl_nr_wait_func,
    ioctl_nr_flip,
    ioctl_nr_demux_op,
    ioctl_nr_acquire_overlay,
    ioctl_nr_release_overlay,
    ioctl_nr_set_overlay,
    ioctl_nr_overlay_flip,
    ioctl_nr_set_frc,
    ioctl_nr_cec_ctl,
    ioctl_nr_wait_event,
    ioctl_nr_set_vcp_flag,
    ioctl_nr_set_shrink,
    ioctl_nr_get_shrink,
    ioctl_nr_set_boot_noise_patch_end,
    ioctl_nr_skip_force_crt_on
};

#define S3G_IOCTL_BASE             'd'
#define S3G_IO(nr)                 _IO(S3G_IOCTL_BASE, nr)
#define S3G_IOR(nr, type)          _IOR(S3G_IOCTL_BASE, nr, type)
#define S3G_IOW(nr, type)          _IOW(S3G_IOCTL_BASE, nr, type)
#define S3G_IOWR(nr, type)         _IOWR(S3G_IOCTL_BASE, nr, type)

#define S3G_IOCTL_GET_BUSID        S3G_IOWR(ioctl_nr_getbusid, s3g_unique_t)
#define S3G_IOCTL_GET_DISPLAYINFO  S3G_IOWR(ioctl_nr_get_displayinfo, s3g_display_info_t)
#define S3G_IOCTL_GET_IGAINFO      S3G_IOWR(ioctl_nr_get_igainfo, s3g_iga_info_t)
#define S3G_IOCTL_ADD_CTX          S3G_IOWR(ioctl_nr_addctx, s3g_ctx_t)
#define S3G_IOCTL_RM_CTX           S3G_IOWR(ioctl_nr_rmctx, s3g_ctx_t)
#define S3G_IOCTL_LOCK             S3G_IOWR(ioctl_nr_lock, s3g_lock_t)
#define S3G_IOCTL_UNLOCK           S3G_IOWR(ioctl_nr_unlock, s3g_lock_t)
#define S3G_IOCTL_CLIP_NOTIFY      S3G_IOWR(ioctl_nr_clipnotify, s3g_clipnotify_t)
#define S3G_IOCTL_ADD_DRAW         S3G_IOWR(ioctl_nr_adddraw, s3g_drawable_t)
#define S3G_IOCTL_RM_DRAW          S3G_IOWR(ioctl_nr_rmdraw, int)
#define S3G_IOCTL_GET_DRAWSIZE     S3G_IOWR(ioctl_nr_getdrawsize, s3g_get_drawsize_t)
#define S3G_IOCTL_QUERY_DISPLAY_CAPS S3G_IOWR(ioctl_nr_query_display_caps, s3g_display_caps_t)
#define S3G_IOCTL_QUERY_OUTPUT_CAPS  S3G_IOWR(ioctl_nr_query_output_caps, s3g_output_caps_t)
#define S3G_IOCTL_MODE_SET         S3G_IOWR(ioctl_nr_mode_set, s3g_mode_info_t)
#define S3G_IOCTL_CURSOR           S3G_IOWR(ioctl_nr_cursor, s3g_set_cursor_t)
#define S3G_IOCTL_GAMMA            S3G_IOWR(ioctl_nr_gamma, s3g_gamma_info_t)
#define S3G_IOCTL_DPMS             S3G_IOWR(ioctl_nr_dpms, s3g_dpms_set_t)
#define S3G_IOCTL_GET_MODE_TIMING  S3G_IOWR(ioctl_nr_get_mode_timing, s3g_mode_timing_info_t)
#define S3G_IOCTL_LEAVE_VT         S3G_IO(ioctl_nr_leave_vt)
#define S3G_IOCTL_ENTER_VT         S3G_IO(ioctl_nr_enter_vt)
#define S3G_IOCTL_CHECK_DEVICE_COMBINATION S3G_IOWR(ioctl_nr_check_device_combination, s3g_check_device_combination_t)
#define S3G_IOCTL_2D_INIT          S3G_IOW(ioctl_nr_2d_init, s3g_2d_init_t)
#define S3G_IOCTL_WAIT_CHIP_IDLE   S3G_IO(ioctl_nr_wait_chip_idle)
#define S3G_IOCTL_ESCAPE_CALL      S3G_IOW(ioctl_nr_escape_call, long)
#define S3G_IOCTL_2D_DRAW          S3G_IOW(ioctl_nr_2d_draw, long)
#define S3G_IOCTL_INIT_SCREEN      S3G_IOW(ioctl_nr_init_screen, s3g_2d_screen_t)
#define S3G_IOCTL_DEINIT_SCREEN    S3G_IOW(ioctl_nr_deinit_screen, s3g_2d_screen_t)
#define S3G_IOCTL_ACQUIRE_APERTURE S3G_IOW(ioctl_nr_acquire_aperture, unsigned int)
#define S3G_IOCTL_RELEASE_APERTURE S3G_IOW(ioctl_nr_release_aperture, unsigned int)
#define S3G_IOCTL_SET_PS           S3G_IOW(ioctl_nr_set_ps, s3g_ps_info_t)
#define S3G_IOCTL_ENUM_SUB_DEVICE  S3G_IOW(ioctl_nr_enum_sub_device, void*)
#define S3G_IOCTL_CREATE_ALLOCATION     S3G_IOWR(ioctl_nr_create_allocation, s3g_create_allocation_t)
#define S3G_IOCTL_DESTROY_ALLOCATION    S3G_IOW(ioctl_nr_destroy_allocation, unsigned int)
#define S3G_IOCTL_LOCK_ALLOCATION       S3G_IOWR(ioctl_nr_lock_allocation, s3g_lock_allocation_t)
#define S3G_IOCTL_UNLOCK_ALLOCATION     S3G_IOW(ioctl_nr_unlock_allocation, s3g_unlock_allocation_t)
#define S3G_IOCTL_WAIT_ALLOCATION_IDLE  S3G_IOW(ioctl_nr_wait_allocation_idle, s3g_wait_allocation_idle_t)
#define S3G_IOCTL_CREATE_OVERLAY        S3G_IOWR(ioctl_nr_create_overlay, s3g_create_overlay_t)
#define S3G_IOCTL_DESTROY_OVERLAY       S3G_IOWR(ioctl_nr_destroy_overlay, unsigned int)
#define S3G_IOCTL_UPDATE_OVERLAY        S3G_IOWR(ioctl_nr_update_overlay, s3g_update_overlay_t)
#define S3G_IOCTL_FLIP_OVERLAY          S3G_IOWR(ioctl_nr_flip_overlay, s3g_flip_overlay_t)
#define S3G_IOCTL_SET_OVERLAY_INFO      S3G_IOWR(ioctl_nr_set_overlay_info, s3g_set_overlay_info_t)
#define S3G_IOCTL_VIDEO_GET_HW_CAPS     S3G_IOWR(ioctl_nr_video_get_hw_caps, unsigned char)
#define S3G_IOCTL_GET_ADDRESS_CONFIG    S3G_IOWR(ioctl_nr_get_address_config, s3g_get_address_config_t)
#define S3G_IOCTL_QUERY_INFO            S3G_IOWR(ioctl_nr_query_info, s3g_query_info_t)
#define S3G_IOCTL_WAIT_FUNC             S3G_IOWR(ioctl_nr_wait_func, s3g_wait_func_t)
#define S3G_IOCTL_FLIP                  S3G_IOW(ioctl_nr_flip, s3g_flip_info_t)
#define S3G_IOCTL_DEMUX_OP              S3G_IOWR(ioctl_nr_demux_op, s3g_demux_op_t)
#define S3G_IOCTL_ACQUIRE_OVERLAY       S3G_IOWR(ioctl_nr_acquire_overlay, s3g_acquire_overlay_t)
#define S3G_IOCTL_RELEASE_OVERLAY       S3G_IOW(ioctl_nr_release_overlay, s3g_release_overlay_t)
#define S3G_IOCTL_SET_OVERLAY           S3G_IOW(ioctl_nr_set_overlay, s3g_set_overlay_t)
#define S3G_IOCTL_OVERLAY_FLIP          S3G_IOW(ioctl_nr_overlay_flip, s3g_overlay_flip_t)
#define S3G_IOCTL_SET_FRC               S3G_IOW(ioctl_nr_set_frc, s3g_set_frc_t)
#define S3G_IOCTL_CEC_CTL               S3G_IOW(ioctl_nr_cec_ctl, s3g_cec_ctl_t)
#define S3G_IOCTL_WAIT_EVENT        S3G_IOW(ioctl_nr_wait_event, s3g_wait_func_t)
#define S3G_IOCTL_SET_EDP_FLAG        S3G_IOW(ioctl_nr_set_vcp_flag, int)
#define S3G_IOCTL_SET_SHRINK            S3G_IOW(ioctl_nr_set_shrink, s3g_set_shrink_t)
#define S3G_IOCTL_GET_SHRINK            S3G_IOW(ioctl_nr_get_shrink, s3g_get_shrink_t)
#define S3G_IOCTL_SET_BOOT_NOISE_PATCH_END  S3G_IOW(ioctl_nr_set_boot_noise_patch_end, int)
#define S3G_IOCTL_SKIP_FORCE_CRT_ON  S3G_IO(ioctl_nr_skip_force_crt_on)


/* for fb Driver private ioctl */
/*s3g + index */
#define FB_S3G_IOCTL_GET_ALLOCATION             0x73336701
#define FB_S3G_IOCTL_SET_CURSOR                 0x73336702
#define FB_S3G_IOCTL_QUEUE_POST_ALLOCATION      0x73336703
#define FB_S3G_IOCTL_DEQUEUE_POST_ALLOCATION    0x73336704
#define FB_S3G_IOCTL_GET_FRAMEBUFFER_INFO       0x73336705
#define FB_S3G_IOCTL_SET_ROTATION               0x73336706

#endif //__S3G_IOCTL_H__

