/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

/*
 * S3G SDK TYPES
 */


#ifndef __S3G_TYPES_H__
#define __S3G_TYPES_H__


typedef enum
{
    S3G_USAGE_DISPLAY_SURFACE   = 0x01,
    S3G_USAGE_OVERLAY_SURFACE   = 0x02,
    S3G_USAGE_CURSOR_SURFACE    = 0x04,
    S3G_USAGE_RENDER_TARGET     = 0x08,
    S3G_USAGE_TEXTURE_BUFFER    = 0x10,
    S3G_USAGE_SHADER_BUFFER     = 0x20,
    S3G_USAGE_DATA_BUFFER       = 0x40,
    S3G_USAGE_TEMP_BUFFER       = 0x80,

}s3g_usage_bit;

typedef enum
{
    S3G_ACCESS_GPU_ONLY   = 1,
    S3G_ACCESS_GPU_ALMOST,
    S3G_ACCESS_CPU_ALMOST,
//    S3G_ACCESS_CPU_ONLY,
}s3g_access_hint;

typedef enum
{
    S3G_FORMAT_A8_UNORM       = 1,
    S3G_FORMAT_B5G6R5_UNORM,
    S3G_FORMAT_B8G8R8A8_UNORM,
    S3G_FORMAT_B8G8R8X8_UNORM,
    S3G_FORMAT_YUY2,
    S3G_FORMAT_NV12,
    S3G_FORMAT_FLOAT32,
    S3G_FORMAT_UINT32,
    S3G_FORMAT_INT32,

}s3g_format;

typedef struct
{
    unsigned int hDevice;            /* in */
    unsigned int hAllocation;        /* in */
    unsigned int fill_pattern;       /* in */
}s3g_fill_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int width;             /* in */
    unsigned int height;            /* in */
    unsigned int usage_mask;        /* in */
    s3g_format   format;            /* in */
    s3g_access_hint access_hint;    /* in */
    unsigned int unpagable : 1;     /* in/out */
    unsigned int tiled     : 1;     /* in/out */

    unsigned int size;              /* out */
    unsigned int bit_cnt;           /* out */
    unsigned int pitch;             /* out */
    unsigned int gbd;               /* out */
    unsigned int tiled_width;       /* out */
    unsigned int gpu_virt_addr;     /* out */
    unsigned int allocation;        /* out */
}s3g_create_allocation_t;


typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */

    unsigned int size;              /* out */
    unsigned int width;             /* out */
    unsigned int height;            /* out */
    unsigned int bit_cnt;           /* out */
    unsigned int pitch;             /* out */
    s3g_format   format;            /* out */
    unsigned int unpagable : 1;     /* out */
    unsigned int tiled     : 1;     /* out */
    unsigned int gpu_virt_addr;     /* out */
}s3g_open_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
}s3g_destroy_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
    unsigned int acquire_apeture :1;/* in */
    unsigned int read_only       :1;/* in */
    unsigned int write_only      :1;/* in */

    void         *virt_addr;        /* out */
}s3g_lock_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
}s3g_unlock_allocation_t;


#endif /* __S3G_TYPES_H__*/

